/*
 * TMCL.c
 *
 *  Created on: Apr 24, 2014
 *      Author: ed
 */
#include "TMCL.h"
#include "tmc//tmcl/TMCL-Compute.h"
#include "StepDir.h"


	u32 TMCLLastError;
	volatile u32 TMCLTickTimer;
	u8 ResetRequested;

	// imported variables
#ifdef BOOTLOADER
	extern u32 BLMagic;
#endif

#ifdef TMCM_USE_BOOT_CONFIG_PAGE
	u32 PageBuffer[FLASH_PAGE_SIZE/4];
	void tmcl_bootWriteConfigPage();
	void tmcl_bootReadConfigPage();
#endif

	// local used functions
	void tmcl_getVersion();
#ifdef BOOTLOADER
	void tmcl_boot();
#endif
	void tmcl_softwareReset();
	void tmcl_resetCPU(u8 resetPeripherals);

	void tmcl_getInput();
	void tmcl_setOutput();

//	void tmcl_writeRegister();
//	void tmcl_readRegister();

	void tmcl_rotateLeft();
	void tmcl_rotateRight();
	void tmcl_motorStop();
	void tmcl_moveToPosition();


/* initialize tmcl */
void tmcl_init()
{
	ResetRequested = FALSE;
	TMCLMode = TM_IDLE;
	TMCLSuppressReply = FALSE;


		TMCLProgramCounter = 0;
		TMCLAccu = 0;
		TMCLXReg = 0;
		TMCLCPUFlags = 0;
		TMCLWaitFlag = false;

		// set all user variables to zero
		u32 i;
		for(i = 0; i < TMCL_RAM_USER_VARS; i++)
			TMCLUserVariable[i] = 0;

		// load user variables from EEPROM if desired
		if(!moduleConfig.ZeroUserVariables)
		{
			for(i = 0; i < TMCL_EEPROM_USER_VARS; i++)
				eeprom_readConfigBlock(ADDR_TMCL_USER_VARS+i*4, (u8 *) &TMCLUserVariable[i], 4);
		}

		// do auto start if desired
		if(moduleConfig.TMCLAutostartFlag)
			TMCLMode = TM_RUN;
		else
			TMCLMode = TM_IDLE;



}

/* execute the TMCL-Command stored in "ActualCommand" */
void tmcl_executeActualCommand()
{
	// prepare reply command
	ActualReply.Opcode = ActualCommand.Opcode;
	ActualReply.Status = REPLY_OK;
	ActualReply.Value.Int32 = ActualCommand.Value.Int32;

	// get command
	switch(ActualCommand.Opcode)
	{
	case TMCL_ROR:
		tmcl_rotateRight();
		break;
	case TMCL_ROL:
		tmcl_rotateLeft();
		break;
	case TMCL_MST:
		tmcl_motorStop();
		break;
	case TMCL_MVP:
		tmcl_moveToPosition();
		break;

	case TMCL_SAP:
	case TMCL_GAP:
	case TMCL_STAP:
	case TMCL_RSAP:

		ActualReply.Status = tmcl_handleAxisParameter(ActualCommand.Motor, ActualCommand.Opcode, ActualCommand.Type, &(ActualCommand.Value.Int32));
		ActualReply.Value.Int32 = ActualCommand.Value.Int32;

		break;
	case TMCL_SGP:
	case TMCL_GGP:
		tmcl_handleGlobalParameter(ActualCommand.Opcode);
		break;
	case TMCL_SIO:
		tmcl_setOutput();
		break;
	case TMCL_GIO:
		tmcl_getInput();
		break;

	case TMCL_CALC:
		Calculate();
		break;
	case TMCL_COMP:
		Compare();
		break;
	case TMCL_JC:
		JumpConditional();
		break;
	case TMCL_JA:
		JumpAlways();
		break;
	case TMCL_CSUB:
		CallSubroutine();
		break;
	case TMCL_RSUB:
		ReturnFromSubroutine();
		break;
	case TMCL_WAIT:
		Wait();
		break;
	case TMCL_STOP:
		Stop();
		break;
	case TMCL_SCO:
		SetCoordinate();
		break;
	case TMCL_GCO:
		GetCoordinate();
		break;
	case TMCL_CCO:
		CaptureCoordinate();
		break;
	case TMCL_CALCX:
		CalculateXRegister();
		break;
	case TMCL_AAP:
		AccuToAxisParameter();
		break;
	case TMCL_AGP:
		AccuToGlobalParameter();
		break;
	case TMCL_CLE:
		ClearError();
		break;
	case TMCL_ACO:
		AccuToCoordinate();
		break;
//	case TMCL_UF0:
//		//tmcl_handleUserFunction(ActualCommand.Opcode);
//		break;
	case TMCL_ApplStop:
		ApplStop();
		break;
	case TMCL_ApplRun:
		ApplRun();
		break;
	case TMCL_ApplStep:
		ApplStep();
		break;
	case TMCL_ApplReset:
		ApplReset();
		break;
	case TMCL_DownloadStart:
		//ApplStop();
		//bldc_setStopRegulationMode();
		DownloadStart(
				moduleConfig.TMCLProtectionMode/*, flags_getAllStatusFlags(ActualCommand.Motor)*/);
		break;
	case TMCL_DownloadEnd:
		DownloadEnd();
//		tmcm_enableDriver();
//		bldc_setTargetMotorCurrent(i, 0);
		break;
	case TMCL_ReadMem:
		ReadTMCLMemory(moduleConfig.SerialHostAddress,
				moduleConfig.TMCLProtectionMode);
		break;
	case TMCL_GetVersion:
		tmcl_getVersion();
		break;
	case TMCL_GetStatus:
		GetApplStatus();
		break;
	case TMCL_FactoryDefault:
		FactoryDefault();
		break;
	case TMCL_SecurityCode:
		SecurityCode();
		break;
	case TMCL_Breakpoint:
		Breakpoint();
		break;

	case TMCL_WriteRegister_Ch1:
		tmc2300_writeInt(&tmc2300, ActualCommand.Type, ActualReply.Value.Int32);
		break;
	case TMCL_ReadRegister_Ch1:
		ActualReply.Value.Int32 = tmc2300_readInt(&tmc2300, ActualCommand.Type);
		break;
#ifdef TMCM_USE_BOOT_CONFIG_PAGE
	case TMCL_BootWriteConfig:
		tmcl_bootWriteConfigPage();
		break;
	case TMCL_BootReadConfig:
		tmcl_bootReadConfigPage();
		break;
#endif
#ifdef BOOTLOADER
	case TMCL_Boot:
		tmcl_boot();
		break;
#endif
	case TMCL_SoftwareReset:
		tmcl_softwareReset();
		break;

	default:
		ActualReply.Status = REPLY_INVALID_CMD;
		break;
	}
}

/* handle communication data */
void tmcl_processCommand(bool allowTMCLScript)
{
	UNUSED(allowTMCLScript);
	static u8 TMCLCommandState;
    u32 i;

#ifdef TMCM_USE_UART_INTERFACE
    u8 Byte;
    static u8 UARTCmd[9];
    static u8 UARTCount;
#endif

    // send answer on last GAP/GGP/GIO/GCO request)
    if(TMCLSuppressReply && ActualCommand.Opcode!=TMCL_GAP && ActualCommand.Opcode!=TMCL_GGP
        && ActualCommand.Opcode!=TMCL_GIO && ActualCommand.Opcode!=TMCL_GCO) TMCLCommandState=TCS_IDLE;



#if defined(TMCM_USE_UART_INTERFACE)

    if(TMCLCommandState==TCS_UART)  //Antwort via UART
    {
      if(TMCLReplyFormat==RF_STANDARD)
      {
        u8 Checksum = moduleConfig.SerialHostAddress+moduleConfig.SerialModuleAddress+
            ActualReply.Status+ActualReply.Opcode+
          ActualReply.Value.Byte[3]+ActualReply.Value.Byte[2]+
          ActualReply.Value.Byte[1]+ActualReply.Value.Byte[0];

        uart_tmcl_write(moduleConfig.SerialHostAddress);
        uart_tmcl_write(moduleConfig.SerialModuleAddress);
        uart_tmcl_write(ActualReply.Status);
        uart_tmcl_write(ActualReply.Opcode);
        uart_tmcl_write(ActualReply.Value.Byte[3]);
        uart_tmcl_write(ActualReply.Value.Byte[2]);
        uart_tmcl_write(ActualReply.Value.Byte[1]);
        uart_tmcl_write(ActualReply.Value.Byte[0]);
        uart_tmcl_write(Checksum);
      }
      else if(TMCLReplyFormat==RF_SPECIAL)
      {
        for(i=0; i<9; i++)
          uart_tmcl_write(SpecialReply[i]);
      }
    }
    else if(TMCLCommandState==TCS_UART_ERROR)  //letztes Kommando via UART hatte falsche checksum
    {
      ActualReply.Opcode = 0;
      ActualReply.Status = REPLY_CHKERR;
      ActualReply.Value.Int32 = 0;

      u8 Checksum = moduleConfig.SerialHostAddress + moduleConfig.SerialModuleAddress+
          ActualReply.Status+ActualReply.Opcode+
          ActualReply.Value.Byte[3] + ActualReply.Value.Byte[2]+
          ActualReply.Value.Byte[1] + ActualReply.Value.Byte[0];

      uart_tmcl_write(moduleConfig.SerialHostAddress);
      uart_tmcl_write(moduleConfig.SerialModuleAddress);
      uart_tmcl_write(ActualReply.Status);
      uart_tmcl_write(ActualReply.Opcode);
      uart_tmcl_write(ActualReply.Value.Byte[3]);
      uart_tmcl_write(ActualReply.Value.Byte[2]);
      uart_tmcl_write(ActualReply.Value.Byte[1]);
      uart_tmcl_write(ActualReply.Value.Byte[0]);
      uart_tmcl_write(Checksum);
    }

#endif

    // Zustand zur�cksetzen (Antwort ist nun gesendet)
    TMCLCommandState = TCS_IDLE;
    TMCLReplyFormat = RF_STANDARD;

    // Letzter Befehl war Reset-Befehl => Reset
    if(ResetRequested)
    	tmcl_resetCPU(true);

    // get a new command: priority: CAN, UART, memory
    bool newCommandHandled = false;



#ifdef TMCM_USE_UART_INTERFACE

    if(uart_tmcl_read((char *)&Byte))  // new UART request available?
    {
      if(uart_tmcl_checkTimeout())
        UARTCount=0;  //discard everything when there has been a command timeout

      UARTCmd[UARTCount++] = Byte;

      if(UARTCount==9)  //Nine bytes have been received without timeout
      {
        UARTCount=0;
        newCommandHandled = true;
//        systemInfo_incCommunicationLoopCounter();

        if(UARTCmd[0] == moduleConfig.SerialModuleAddress)  // is this our address?
        {
          u8 Checksum = 0;
          for(i=0; i<8; i++)
            Checksum+=UARTCmd[i];

          if(Checksum==UARTCmd[8])  // is the checksum correct?
          {
            ActualCommand.Opcode=UARTCmd[1];
            ActualCommand.Type=UARTCmd[2];
            ActualCommand.Motor=UARTCmd[3];
            ActualCommand.Value.Byte[3]=UARTCmd[4];
            ActualCommand.Value.Byte[2]=UARTCmd[5];
            ActualCommand.Value.Byte[1]=UARTCmd[6];
            ActualCommand.Value.Byte[0]=UARTCmd[7];

            SerialHeartbeatTimer = systick_getTimer();
            SerialHeartbeatFlag = false;

            TMCLCommandState = TCS_UART;

            if(TMCLMode!=TM_RUN && TMCLMode!=TM_STEP && TMCLMode!=TM_DEBUG)
              TMCLModifyAccu = true;
            else
              TMCLModifyAccu = false;
          }
          else TMCLCommandState = TCS_UART_ERROR;  //Checksum wrong
        }
      }
    }
#endif

    if ((newCommandHandled == false) && (TMCLMode==TM_RUN || TMCLMode==TM_STEP)) // Befehl aus Speicher, nur wenn im Run-Modus
    {
      if(TMCLProgramCounter<=TMCL_MEM_SIZE)
      {
        eeprom_readTMCLCommand(TMCLProgramCounter++, &ActualCommand);
        TMCLCommandState=TCS_MEM;

        TMCLModifyAccu=true;
      }
      else TMCLMode=TM_IDLE;
    }

    // handle tmcl command or save it when actual mode is download mode
    if(TMCLMode != TM_DOWNLOAD || ActualCommand.Opcode > 127)  // Commands >= 128 can not be saved!
    {
      // Befehl ausf�hren wenn erfolgreich ein Befehl geholt wurde.
      if(TMCLCommandState!=TCS_IDLE && TMCLCommandState!=TCS_UART_ERROR &&
          TMCLCommandState!=TCS_USB_ERROR && TMCLCommandState!=TCS_UART_II_ERROR) tmcl_executeActualCommand();

      // Im Single-Step-Modus danach Programm wieder anhalten.
      if(TMCLCommandState==TCS_MEM && TMCLMode==TM_STEP) TMCLMode=TM_DEBUG;

      // Auf Breakpoint pr�wenn wir im Run-Modus sind.
      if((TMCLBreakpointCount > 0) && (TMCLCommandState==TCS_MEM) && (TMCLMode==TM_RUN))
      {
        for(i = 0; i < TMCLBreakpointCount; i++)
        {
          if(TMCLProgramCounter == TMCLBreakpoint[i])  //Stehen wir an einem Breakpoint?
          {
            TMCLMode = TM_DEBUG;  //Programm anhalten.
            break;
          }
        }
      }
    }
    else
    {
      // download-mode and Opcode<127

      // store the tmcl command
      if(TMCLCommandState!=TCS_IDLE && TMCLCommandState!=TCS_UART_ERROR &&
          TMCLCommandState!=TCS_USB_ERROR && TMCLCommandState!=TCS_UART_II_ERROR) StoreActualCommand();
    }
}

#define READ   0
#define WRITE  1


//void tmcl_handleAxisParameter(u8 command){};


uint32_t tmcl_handleAxisParameter(uint8_t motor, uint8_t command, uint8_t type, int32_t *value)

{
	uint32_t errors = TMC_ERROR_NONE;
	int32_t buffer = 0;

	uint8_t readWrite = command == TMCL_GAP?READ:WRITE;

//	uint32_t* value = &ActualCommand.Value.Int32;

//	uint8_t motor = ActualCommand.Motor;

	if(motor == 0)
	{
		// \xml <axis_parameters axis="0">
		switch(type)
		{

			case 0:
				// Target position
				if(readWrite == READ) {
					*value = StepDir_getTargetPosition(motor);
				} else if(readWrite == WRITE) {
					StepDir_moveTo(motor, *value);
				}
				break;
			case 1:
				// Actual position
				if(readWrite == READ) {
					*value = StepDir_getActualPosition(motor);
				} else if(readWrite == WRITE) {
					StepDir_setActualPosition(motor, *value);
				}
				break;
			case 2:
				// Target speed
				if(readWrite == READ) {
					*value = StepDir_getTargetVelocity(motor);
				} else if(readWrite == WRITE) {
					StepDir_rotate(motor, *value);
				}
				break;
			case 3:
				// Actual speed
				if(readWrite == READ) {
					*value = StepDir_getActualVelocity(motor);
				} else if(readWrite == WRITE) {
					return REPLY_INVALID_VALUE;
				}
				break;
			case 4:
				// Maximum speed
				if(readWrite == READ) {
					*value = StepDir_getVelocityMax(motor);
				} else if(readWrite == WRITE) {
					StepDir_setVelocityMax(motor, abs(*value));
				}
				break;
			case 5:
				// Maximum acceleration
				if(readWrite == READ) {
					*value = StepDir_getAcceleration(motor);
				} else if(readWrite == WRITE) {
					StepDir_setAcceleration(motor, *value);
				}
				break;
			case 6:
				// Maximum current
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_IHOLD_IRUN, TMC2300_IRUN_MASK, TMC2300_IRUN_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_IHOLD_IRUN, TMC2300_IRUN_MASK, TMC2300_IRUN_SHIFT, *value);
				}
				break;
			case 7:
				// Standby current
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_IHOLD_IRUN, TMC2300_IHOLD_MASK, TMC2300_IHOLD_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_IHOLD_IRUN, TMC2300_IHOLD_MASK, TMC2300_IHOLD_SHIFT, *value);
				}
				break;
			case 8:
				// Position reached flag
				if(readWrite == READ) {
					*value = (StepDir_getStatus(motor) & STATUS_TARGET_REACHED)? 1:0;
				} else if(readWrite == WRITE) {
					return REPLY_INVALID_VALUE;
				}
				break;
			case 29:
				// Measured Speed
				if(readWrite == READ) {
					buffer = (int32_t)(((int64_t)StepDir_getFrequency(motor) * (int64_t)122) / (int64_t)TMC2300_FIELD_READ(&tmc2300, TMC2300_TSTEP, TMC2300_TSTEP_MASK, TMC2300_TSTEP_SHIFT));
					*value = (abs(buffer) < 20) ? 0 : buffer;
				} else if(readWrite == WRITE) {
					return REPLY_INVALID_VALUE;
				}
				break;
			case 100:
				if(readWrite == READ) {
					*value = CAST_Sn_TO_S32(tmc2300_readInt(&tmc2300, TMC2300_VACTUAL), 24);;
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_VACTUAL, TMC2300_VACTUAL_MASK, TMC2300_VACTUAL_SHIFT, *value);
				}
				break;

			case 140:
				// Microstep Resolution
				if(readWrite == READ) {
					*value = 256 >> TMC2300_FIELD_READ(&tmc2300, TMC2300_CHOPCONF, TMC2300_MRES_MASK, TMC2300_MRES_SHIFT);
				} else if(readWrite == WRITE) {
					switch(*value)
					{
					case 1:    *value = 8;   break;
					case 2:    *value = 7;   break;
					case 4:    *value = 6;   break;
					case 8:    *value = 5;   break;
					case 16:   *value = 4;   break;
					case 32:   *value = 3;   break;
					case 64:   *value = 2;   break;
					case 128:  *value = 1;   break;
					case 256:  *value = 0;   break;
					default:   *value = -1;  break;
					}

					if(*value != -1)
					{
						TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_CHOPCONF, TMC2300_MRES_MASK, TMC2300_MRES_SHIFT, *value);
					}
					else
					{
						errors |= TMC_ERROR_VALUE;
					}
				}
				break;
			case 141:
				// Chopper blank time
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_CHOPCONF, TMC2300_INTPOL_MASK, TMC2300_INTPOL_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_CHOPCONF, TMC2300_INTPOL_MASK, TMC2300_INTPOL_SHIFT, *value);
				}
				break;
			case 162:
				// Chopper blank time
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_CHOPCONF, TMC2300_TBL_MASK, TMC2300_TBL_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_CHOPCONF, TMC2300_TBL_MASK, TMC2300_TBL_SHIFT, *value);
				}
				break;
			case 168:
				// smartEnergy current minimum (SEIMIN)
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_COOLCONF, TMC2300_SEIMIN_MASK, TMC2300_SEIMIN_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_COOLCONF, TMC2300_SEIMIN_MASK, TMC2300_SEIMIN_SHIFT, *value);
				}
				break;
			case 169:
				// smartEnergy current down step
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_COOLCONF, TMC2300_SEDN_MASK, TMC2300_SEDN_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_COOLCONF, TMC2300_SEDN_MASK, TMC2300_SEDN_SHIFT, *value);
				}
				break;
			case 170:
				// smartEnergy hysteresis
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_COOLCONF, TMC2300_SEMAX_MASK, TMC2300_SEMAX_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_COOLCONF, TMC2300_SEMAX_MASK, TMC2300_SEMAX_SHIFT, *value);
				}
				break;
			case 171:
				// smartEnergy current up step
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_COOLCONF, TMC2300_SEUP_MASK, TMC2300_SEUP_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_COOLCONF, TMC2300_SEUP_MASK, TMC2300_SEUP_SHIFT, *value);
				}
				break;
			case 172:
				// smartEnergy hysteresis start
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_COOLCONF, TMC2300_SEMIN_MASK, TMC2300_SEMIN_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_COOLCONF, TMC2300_SEMIN_MASK, TMC2300_SEMIN_SHIFT, *value);
				}
				break;
			case 174:
				// stallGuard2 threshold
				if(readWrite == READ) {
					//*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_COOLCONF, TMC2300_SGT_MASK, TMC2300_SGT_SHIFT);
					//*value = StepDir_getStallGuardThreshold(motor);
					*value = tmc2300_readInt(&tmc2300, TMC2300_SGTHRS);
					//*value = CAST_Sn_TO_S32(*value, 7);
				} else if(readWrite == WRITE) {
					tmc2300_writeInt(&tmc2300, TMC2300_SGTHRS, *value);
					//StepDir_setStallGuardThreshold(motor, *value);
					//TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_COOLCONF, TMC2300_SGT_MASK, TMC2300_SGT_SHIFT, *value);
				}
				break;
			case 180:
				// smartEnergy actual current
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_DRVSTATUS, TMC2300_CS_ACTUAL_MASK, TMC2300_CS_ACTUAL_SHIFT);
				} else if(readWrite == WRITE) {
					return REPLY_INVALID_VALUE;
				}
				break;
			case 181:
				// smartEnergy stall velocity
				if(readWrite == READ) {
					*value = StepDir_getStallGuardThreshold(motor);
				} else if(readWrite == WRITE) {
					// Store the threshold value in the internal StepDir generator
					StepDir_setStallGuardThreshold(motor, *value);

					// Convert the value for the TCOOLTHRS register
					// The IC only sends out Stallguard errors while TCOOLTHRS >= TSTEP >= TPWMTHRS
					// The TSTEP value is measured. To prevent measurement inaccuracies hiding
					// a stall signal, we decrease the needed velocity by roughly 12% before converting it.
					*value -= (*value) >> 3;
					if (*value)
					{
						*value = MIN(0x000FFFFF, (1<<24) / (*value));
					}
					else
					{
						*value = 0x000FFFFF;
					}
					tmc2300_writeInt(&tmc2300, TMC2300_TCOOLTHRS, *value);
				}
				break;
			case 182:
				// smartEnergy threshold speed
				if(readWrite == READ) {
					buffer = tmc2300_readInt(&tmc2300, TMC2300_TCOOLTHRS);
					*value = MIN(0xFFFFF, (1<<24) / ((buffer) ? buffer : 1));
				} else if(readWrite == WRITE) {
					*value = MIN(0xFFFFF, (1<<24) / ((*value) ? *value : 1));
					tmc2300_writeInt(&tmc2300, TMC2300_TCOOLTHRS, *value);
				}
				break;
			case 187:
				// PWM gradient
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_PWMCONF, TMC2300_PWM_GRAD_MASK, TMC2300_PWM_GRAD_SHIFT);
				} else if(readWrite == WRITE) {
					// Set gradient
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_PWMCONF, TMC2300_PWM_GRAD_MASK, TMC2300_PWM_GRAD_SHIFT, *value);

					// Enable/disable stealthChop accordingly
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_GCONF, TMC2300_EN_SPREADCYCLE_MASK, TMC2300_EN_SPREADCYCLE_SHIFT, (*value > 0) ? 0 : 1);
				}
				break;
			case 191:
				// PWM frequency
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_PWMCONF, TMC2300_PWM_FREQ_MASK, TMC2300_PWM_FREQ_SHIFT);
				} else if(readWrite == WRITE) {
					if(*value >= 0 && *value < 4)
					{
						TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_PWMCONF, TMC2300_PWM_FREQ_MASK, TMC2300_PWM_FREQ_SHIFT, *value);
					}
					else
					{
						errors |= TMC_ERROR_VALUE;
					}
				}
				break;
			case 192:
				// PWM autoscale
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_PWMCONF, TMC2300_PWM_AUTOSCALE_MASK, TMC2300_PWM_AUTOSCALE_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_PWMCONF, TMC2300_PWM_AUTOSCALE_MASK, TMC2300_PWM_AUTOSCALE_SHIFT, (*value)? 1:0);
				}
				break;
			case 204:
				// Freewheeling mode
				if(readWrite == READ) {
					*value = TMC2300_FIELD_READ(&tmc2300, TMC2300_PWMCONF, TMC2300_FREEWHEEL_MASK, TMC2300_FREEWHEEL_SHIFT);
				} else if(readWrite == WRITE) {
					TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_PWMCONF, TMC2300_FREEWHEEL_MASK, TMC2300_FREEWHEEL_SHIFT, *value);
				}
				break;
			case 206:
				// Load value
				if(readWrite == READ) {
					*value = tmc2300_readInt(&tmc2300, TMC2300_SG_VALUE);
				} else if(readWrite == WRITE) {
					return REPLY_INVALID_VALUE;
				}
				break;

			case 220:
				// Load value
				if(readWrite == READ) {
					*value = tmcm_getModuleVoltage();
				} else if(readWrite == WRITE) {
					return REPLY_INVALID_VALUE;
				}
				break;


		default:
			return REPLY_WRONG_TYPE;
			break;
		}
	} else {
		return REPLY_INVALID_VALUE;
	}

	if (command == TMCL_GAP)
	{
		if(TMCLModifyAccu)
		{
			TMCLAccu = *value;

			if(TMCLAccu == 0)
				TMCLCPUFlags |= FLAG_ZERO;
			else
				TMCLCPUFlags &= ~FLAG_ZERO;
		}
	}

	return REPLY_OK;
}


/* TMCL command GGP, SGP, STGP, ... */
void tmcl_handleGlobalParameter(u8 command) {
	if (ActualCommand.Motor == 0)	// module config values
			{
		switch (ActualCommand.Type) {
//			case 64:
//				// this parameter should be undocumented!
//				if (command == TMCL_GGP) {
//					ActualReply.Value.Int32 = eeprom_readConfigByte(TMCM_ADDR_EEPROM_MAGIC);
//				} else if (command == TMCL_SGP) {
//    				eeprom_writeConfigByte(TMCM_ADDR_EEPROM_MAGIC, ActualCommand.Value.Byte[0]);
//				}
//				break;
		case 65:
			if (command == TMCL_GGP) {
				ActualReply.Value.Int32 = moduleConfig.Baudrate;
			} else if (command == TMCL_SGP) {
				moduleConfig.Baudrate = ActualCommand.Value.Byte[0];
				eeprom_writeConfigByte(
				TMCM_ADDR_MODULE_CONFIG + (u32) &moduleConfig.Baudrate - (u32) &moduleConfig, moduleConfig.Baudrate);
			}
			break;
		case 77: // auto start mode
			if (command == TMCL_GGP) {
				ActualReply.Value.Int32 = moduleConfig.TMCLAutostartFlag;
			} else if (command == TMCL_SGP) {
				moduleConfig.TMCLAutostartFlag = ActualCommand.Value.Byte[0];
				eeprom_writeConfigByte(
				TMCM_ADDR_MODULE_CONFIG + (u32) &moduleConfig.TMCLAutostartFlag - (u32) &moduleConfig,
						moduleConfig.TMCLAutostartFlag);
			}
			break;

		case 81:
			if (command == TMCL_GGP) {
				ActualReply.Value.Int32 = moduleConfig.TMCLProtectionMode;
			} else if (command == TMCL_SGP) {
				// TMCL-memory will be cleared on reset of readout protection
				if ((moduleConfig.TMCLProtectionMode & PB_READ) && !(ActualCommand.Value.Byte[0] & PB_READ)) {
					eeprom_eraseTMCLProgram();
				}
				moduleConfig.TMCLProtectionMode = ActualCommand.Value.Byte[0];
				eeprom_writeConfigByte(
				TMCM_ADDR_MODULE_CONFIG + (u32) &moduleConfig.TMCLProtectionMode - (u32) &moduleConfig,
						moduleConfig.TMCLProtectionMode);
			}
			break;

		case 85: // clear user variables
			if (command == TMCL_GGP) {
				ActualReply.Value.Int32 = moduleConfig.ZeroUserVariables;
			} else if (command == TMCL_SGP) {
				if (ActualCommand.Value.Int32 == 0 || ActualCommand.Value.Int32 == 1) {
					moduleConfig.ZeroUserVariables = ActualCommand.Value.Byte[0];
					eeprom_writeConfigBlock(
					TMCM_ADDR_MODULE_CONFIG + (u32) &moduleConfig.ZeroUserVariables - (u32) &moduleConfig,
							(u8*) &moduleConfig.ZeroUserVariables, sizeof(moduleConfig.ZeroUserVariables));
				} else
					ActualReply.Status = REPLY_INVALID_VALUE;
			}
			break;
		case 128: // application status
			if (command == TMCL_GGP)
				ActualReply.Value.Int32 = TMCLMode;
			break;
		case 129:
			if (command == TMCL_GGP)
				ActualReply.Value.Int32 = (TMCLMode == TM_DOWNLOAD) ? 1 : 0;
			break;
		case 130: // program counter
			if (command == TMCL_GGP)
				ActualReply.Value.Int32 = TMCLProgramCounter;
			break;
		case 131: //131..255: other global parameter
			if (command == TMCL_GGP) {
				ActualReply.Value.Int32 = TMCLLastError;
			} else if (command == TMCL_SGP) {
				TMCLLastError = ActualReply.Value.Int32;
			}
			break;
		case 132: // tick timer
			if (command == TMCL_GGP) {
				ActualReply.Value.Int32 = TMCLTickTimer;
			} else if (command == TMCL_SGP) {
				TMCLTickTimer = ActualReply.Value.Int32;
			}
			break;
		case 255:
			if (command == TMCL_GGP) {
				ActualReply.Value.Int32 = TMCLSuppressReply ? 1 : 0;
			} else if (command == TMCL_SGP) {
				TMCLSuppressReply = ActualCommand.Value.Byte[0];
			}
			break;
		default:
			ActualReply.Status = REPLY_WRONG_TYPE;
			break;
		}
	} else if (ActualCommand.Motor == 2) // TMCL user variables
			{
		if (ActualCommand.Type < TMCL_EEPROM_USER_VARS) // TMCL user variables (0..55)
		{
			if (command == TMCL_GGP) {
				ActualReply.Value.Int32 = TMCLUserVariable[ActualCommand.Type];
			} else if (command == TMCL_SGP) {
				TMCLUserVariable[ActualCommand.Type] = ActualCommand.Value.Int32;
			} else if (command == TMCL_STGP) {
				eeprom_writeConfigBlock(ADDR_TMCL_USER_VARS + ActualCommand.Type * 4,
						(u8*) &TMCLUserVariable[ActualCommand.Type], 4);
			} else if (command == TMCL_RSGP) {
				eeprom_readConfigBlock(ADDR_TMCL_USER_VARS + ActualCommand.Type * 4,
						(u8*) &TMCLUserVariable[ActualCommand.Type], 4);
			}
		}
	} else {
		ActualReply.Status = REPLY_INVALID_VALUE;
	}

	  if (command == TMCL_GGP)
	  {
	    if (TMCLModifyAccu)
	      {
	      TMCLAccu = ActualReply.Value.Int32;

	        if(TMCLAccu == 0)
	          TMCLCPUFlags |= FLAG_ZERO;
	        else
	          TMCLCPUFlags &= ~FLAG_ZERO;
	      }
	  }
}

/* TMCL command SIO */
void tmcl_setOutput()
{
	switch(ActualCommand.Motor)
	{
    	case 2:
    		if(ActualCommand.Value.Int32 == 0)
    			tmcm_clearModuleSpecificIOPin(ActualCommand.Type);
    		else
    			tmcm_setModuleSpecificIOPin(ActualCommand.Type);
    		break;
    	default:
    		ActualReply.Status = REPLY_INVALID_VALUE;
    		break;
	}
}

/* TMCL command GIO */
void tmcl_getInput()
{
	switch(ActualCommand.Motor)
	{
		case 0:
			ActualReply.Value.Int32 = tmcm_getModuleSpecificIOPin(ActualCommand.Type);
    		break;
//    	case 1:
//    		ActualReply.Value.Int32 = tmcm_getModuleSpecificADCValue(ActualCommand.Type);
//    		break;
    	case 2:
    		ActualReply.Value.Int32 = tmcm_getModuleSpecificIOPinStatus(ActualCommand.Type);
    		break;
    	default:
    		ActualReply.Status = REPLY_INVALID_VALUE;
    		break;
	}
	if (ActualCommand.Opcode == TMCL_GIO)
	{
		if(TMCLModifyAccu)
		{
			TMCLAccu = ActualReply.Value.Int32;

			if(TMCLAccu==0)
				TMCLCPUFlags |= FLAG_ZERO;
			else
				TMCLCPUFlags &= ~FLAG_ZERO;
		}
	}
}

/* TMCL command ROL */
void tmcl_rotateLeft()
{
	StepDir_rotate(0, -ActualCommand.Value.Int32);
}

/* TMCL command ROR */
void tmcl_rotateRight()
{
	StepDir_rotate(0, ActualCommand.Value.Int32);
}

/* TMCL command MST */
void tmcl_motorStop()
{
	StepDir_rotate(0, 0);
}

/* TMCL command MVP */
void tmcl_moveToPosition()
{
	if(ActualCommand.Motor == 0)
	{
		switch(ActualCommand.Type)
		{
		case MVP_ABS:
			StepDir_moveTo(0, ActualCommand.Value.Int32);
			break;
		case MVP_REL:
			ActualCommand.Value.Int32 += StepDir_getActualPosition(0);
			StepDir_moveTo(0, ActualCommand.Value.Int32);
			break;
		default:
			ActualReply.Status = REPLY_WRONG_TYPE;
			break;
		}
	} else ActualReply.Status = REPLY_INVALID_VALUE;
}

/* TMCL command 136 (Get Version) */
void tmcl_getVersion()
{
	if(ActualCommand.Type == 0)
	{
		TMCLReplyFormat = RF_SPECIAL;
		SpecialReply[0] = moduleConfig.SerialHostAddress;
		u32 i;
		for(i=0; i<8; i++)
			SpecialReply[i+1] = VersionString[i];
	}
	else if(ActualCommand.Type == 1)
	{
		ActualReply.Value.Byte[3] = SW_TYPE_HIGH;
		ActualReply.Value.Byte[2] = SW_TYPE_LOW;
		ActualReply.Value.Byte[1] = SW_VERSION_HIGH;
		ActualReply.Value.Byte[0] = SW_VERSION_LOW;
	}
#if defined(BOOTLOADER)
	else if (ActualCommand.Type == 2)
	{
		TMCLReplyFormat = RF_SPECIAL;
		u8 *bootloaderInfo = ((u8 *)&__vect_table)-8;
		SpecialReply[0] = moduleConfig.SerialHostAddress;
		u32 i;
		for(i=0; i<8; i++)
			SpecialReply[i+1] = bootloaderInfo[i];
	}
#endif
}


// todo: nach Cpu.h/c
void tmcl_resetCPU(u8 resetPeripherals)
{
	UNUSED(resetPeripherals);
	SCB_AIRCR = SCB_AIRCR_VECTKEY(0x5FA) | SCB_AIRCR_SYSRESETREQ_MASK;
}

#ifdef BOOTLOADER
/* special command to switch to bootloader mode */
void tmcl_boot()
{
	if(ActualCommand.Type==0x81 && ActualCommand.Motor==0x92 &&
			ActualCommand.Value.Byte[3]==0xa3 && ActualCommand.Value.Byte[2]==0xb4 &&
			ActualCommand.Value.Byte[1]==0xc5 && ActualCommand.Value.Byte[0]==0xd6)
	{
		force_bootloader();
	}
	else
	{
		ActualReply.Status=REPLY_INVALID_VALUE;
	}
}

void force_bootloader(){
	tmcm_disable_driver();

	DisableInterrupts;
	systick_deInit();
    NVIC_ICER=0xFFFFFFFF;  //alle Interrupts sperren
    NVIC_ICPR=0xFFFFFFFF;
    BLMagic=0x12345678;
    tmcl_resetCPU(TRUE);
}

#endif

/* TMCL command 255 (Reset) */
void tmcl_softwareReset()
{
	if(ActualCommand.Value.Int32==1234)
		ResetRequested = TRUE;
}
