/*
 * SPI.c
 *
 *  Created on: 17.03.2016
 *      Author: bs
 */

#include "SPI.h"
#include "TMCL.h"

#define SPISR_SPTEF(base) ((SPI_S_REG(base) & SPI_S_SPTEF_MASK))
#define SPISR_SPRF(base)  ((SPI_S_REG(base) & SPI_S_SPRF_MASK))

void spi1_init()
{
	volatile u8 dummy;

	/* enable SPI clocks */
	SIM_SCGC |= SIM_SCGC_SPI0_MASK;

	SPI_MemMapPtr base = SPI0_BASE_PTR;

	SPI_C1_REG(base) = SPI_C1_MSTR_MASK | SPI_C1_CPOL_MASK | SPI_C1_CPHA_MASK; // master, clock polarity and clock phase

	/* // Automatic Chip Select (not needed, done manually)
	SPI_C1_REG(base) |= SPI_C1_SSOE_MASK;
	SPI_C2_REG(base) |= SPI_C2_MODFEN_MASK;
	 */

	SPI_BR_REG(base) = SPI_BR_SPPR(1) | SPI_BR_SPR(1);
	// BaudRateDivisor = (SPPR + 1) � 2^(SPR + 1) = 8
	// BaudRate = BusClock (16 MHz) / BaudRateDivisor = 2 MHz

	dummy = SPI_D_REG(base);
	UNUSED(dummy);

	SPI_C2_REG(base) = 0;

	SPI_C1_REG(base) |= SPI_C1_SPE_MASK; // enable SPI
}

u8 spi_readWrite(u8 deviceNumber, u8 writeData, u8 lastTransfer)
{
	UNUSED(deviceNumber);

	u8 readData = 0;
	SPI_MemMapPtr base = SPI0_BASE_PTR;

	tmcm_clear_SPI_CS();

	while (!SPISR_SPTEF(base));   /* sending finish? */
	SPI_D_REG(base) = writeData;

	while(!SPISR_SPRF(base)); /* read finish? */
	readData = SPI_D_REG(base);

	//Set CS-Signal back high if lastTransfer
	if(lastTransfer)
	{
		tmcm_set_SPI_CS();

	}

	return readData;
}

void eeprom_initSpi()
{
	spi1_init();
}

/* read and write spi data */
u8 eeprom_readWriteByte(u8 data, u8 lastTransfer)
{
	return spi_readWrite(0, data, lastTransfer);
}

/* initialize the eeprom and read the default parameters from eeprom */
void eeprom_initConfig(/*TModuleConfig *moduleConfigL*//*, TMotorConfig *motorConfigL*/)
{
	// initialize the eeprom
	if (eeprom_readConfigByte(TMCM_ADDR_EEPROM_MAGIC) != TMCM_EEPROM_MAGIC || ResetToFactoryDefaultRequested)
	{
		eeprom_writeConfigByte(TMCM_ADDR_EEPROM_MAGIC, 0);

		// clear tmcl area from (2048..16383)
		eeprom_eraseTMCLProgram();

		// overwrite module configuration
		eeprom_writeConfigBlock(TMCM_ADDR_MODULE_CONFIG, (u8 *)&moduleConfig, sizeof(TModuleConfig));

		// overwrite motor configuration

//		eeprom_writeConfigBlock(TMCM_ADDR_MOTOR_CONFIG, (u8 *)&motorConfig, sizeof(TMotorConfig)/*TMCM_MOTOR_CONFIG_SIZE*/);

		// update magic byte
		eeprom_writeConfigByte(TMCM_ADDR_EEPROM_MAGIC, TMCM_EEPROM_MAGIC);

		if(ResetToFactoryDefaultRequested){
#ifdef BOOTLOADER
			force_bootloader();
#else
			tmcm_setModuleSpecificIOPin(1);
#endif
		}
	}

	// read the default module configuration from EEPROM
	eeprom_readConfigBlock(TMCM_ADDR_MODULE_CONFIG, (u8 *)&moduleConfig, sizeof(TModuleConfig));

	// read the default motor configurations from EEPROM
//	eeprom_readConfigBlock(TMCM_ADDR_MOTOR_CONFIG, (u8 *)&motorConfig, sizeof(TMotorConfig)/*TMCM_MOTOR_CONFIG_SIZE*/);

}

/* read a byte from the configuration eeprom (address: 0..2047) */
u8 eeprom_readConfigByte(u32 address)
{
	eeprom_readWriteByte(0x03, false);  //Befehl "Read"
	eeprom_readWriteByte(address >> 8, false);
	eeprom_readWriteByte(address & 0xff, false);
	return eeprom_readWriteByte(0, true);
}

/* write a byte into the configuration eeprom (address: 0..2047) */
void eeprom_writeConfigByte(u32 address, u8 value)
{
	// enable writing
	eeprom_readWriteByte(0x06, true);  // command "Write Enable"
	do
	{
		eeprom_readWriteByte(0x05, false);  // command "Get Status"
	} while((eeprom_readWriteByte(0x00, true) & 0x02) == 0x00);  // wait until "Write Enable"-Bit is set

	// write
	eeprom_readWriteByte(0x02, false); // command "Write"
	eeprom_readWriteByte(address >> 8, false);
	eeprom_readWriteByte(address & 0xff, false);
	eeprom_readWriteByte(value, true);

	// wait until write is finished
	do
	{
		eeprom_readWriteByte(0x05, false);  // command "Get Status"
	} while(eeprom_readWriteByte(0x00, true) & 0x01);
}

/* erase the complete TMCL program */
void eeprom_eraseTMCLProgram()
{
	u32 address;
	for(address = TMCM_ADDR_TMCL_PROG; address < 16384; address += 64)
	{
		// enable writing
		eeprom_readWriteByte(0x06, true);  // command "Write Enable"
		do
		{
			eeprom_readWriteByte(0x05, false);  // command "Get Status"
		} while((eeprom_readWriteByte(0x00, true) & 0x02)==0x00);  // wait until "Write Enable"-Bit is set

		// write (start address)
		eeprom_readWriteByte(0x02, false); // command "Write"
		eeprom_readWriteByte(address >> 8, false);
		eeprom_readWriteByte(address & 0xff, false);

		u32 i;
		for(i=0; i<64; i++)
			eeprom_readWriteByte(0, (i==63));

		// wait until write is finished
		do
		{
			eeprom_readWriteByte(0x05, false);  // command "Get Status"
		} while(eeprom_readWriteByte(0x00, true) & 0x01);
	}
}

/* compare the eeprom content at the given address with the TMCL command (return true if both have the same content) */
u32 eeprom_verifyTMCLCommand(u32 address, TTMCLCommand *TMCLCommand)
{
	// TMCL commands are stored from address 2048 with 7 Bytes per command
	address = address*7 + 2048;

	eeprom_readWriteByte(0x03, false);  // command "Read"
	eeprom_readWriteByte(address >> 8, false);
	eeprom_readWriteByte(address & 0xff, false);

	TTMCLCommand Verify;
	Verify.Opcode = eeprom_readWriteByte(0, false);
	Verify.Type = eeprom_readWriteByte(0, false);
	Verify.Motor = eeprom_readWriteByte(0, false);
	Verify.Value.Byte[0] = eeprom_readWriteByte(0, false);
	Verify.Value.Byte[1] = eeprom_readWriteByte(0, false);
	Verify.Value.Byte[2] = eeprom_readWriteByte(0, false);
	Verify.Value.Byte[3] = eeprom_readWriteByte(0, true);

	return Verify.Opcode==TMCLCommand->Opcode && Verify.Type==TMCLCommand->Type &&
         Verify.Motor==TMCLCommand->Motor && Verify.Value.Int32==TMCLCommand->Value.Int32;
}

/* write a TMCL command into the eeprom (address: 0..2047) */
void eeprom_writeTMCLCommand(u32 address, TTMCLCommand *TMCLCommand)
{
	// TMCL commands are stored from address 2048 with 7 Bytes per command
	address = address*7 + 2048;

	eeprom_writeConfigByte(address++, TMCLCommand->Opcode);
	eeprom_writeConfigByte(address++, TMCLCommand->Type);
	eeprom_writeConfigByte(address++, TMCLCommand->Motor);
	eeprom_writeConfigByte(address++, TMCLCommand->Value.Byte[0]);
	eeprom_writeConfigByte(address++, TMCLCommand->Value.Byte[1]);
	eeprom_writeConfigByte(address++, TMCLCommand->Value.Byte[2]);
	eeprom_writeConfigByte(address++, TMCLCommand->Value.Byte[3]);
}

/* read a TMCL command from eeprom */
void eeprom_readTMCLCommand(u32 address, TTMCLCommand *TMCLCommand)
{
	// TMCL commands are stored from address 2048 with 7 Bytes per command
	address = address*7 + 2048;

	eeprom_readWriteByte(0x03, false);  // command "Read"
	eeprom_readWriteByte(address >> 8, false);
	eeprom_readWriteByte(address & 0xff, false);

	TMCLCommand->Opcode = eeprom_readWriteByte(0, false);
	TMCLCommand->Type = eeprom_readWriteByte(0, false);
	TMCLCommand->Motor = eeprom_readWriteByte(0, false);
	TMCLCommand->Value.Byte[0] = eeprom_readWriteByte(0, false);
	TMCLCommand->Value.Byte[1] = eeprom_readWriteByte(0, false);
	TMCLCommand->Value.Byte[2] = eeprom_readWriteByte(0, false);
	TMCLCommand->Value.Byte[3] = eeprom_readWriteByte(0, true);
}

/* write a block into the configuration eeprom (using eeprom 25128),
 * (address: (0..2047) (block: start address of the block) */
void eeprom_writeConfigBlock(u32 address, u8 *block, u32 size)
{
	// enable reading
	eeprom_readWriteByte(0x06, true);  // command "Write Enable"
	do
	{
		eeprom_readWriteByte(0x05, false);  // command "Get Status"
	} while((eeprom_readWriteByte(0x00, true) & 0x02) == 0x00);  // wait until "Write Enable"-Bit is set

	// write (Startadresse)
	eeprom_readWriteByte(0x02, false); //Befehl "Write"
	eeprom_readWriteByte(address >> 8, false);
	eeprom_readWriteByte(address & 0xff, false);

	// write data
	u32 i;
	for(i=0; i<size; i++)
	{
		//Adresse mitz�hlen und bei �berlauf der untersten sechs Bits das EEPROM deselektieren
		//und neuen Write-Befehl senden (bzw. beim letzten Datenbyte einfach nur EEPROM
		//deselektieren).
		//Dies ist erforderlich, da beim Beschreiben im 25128 nur die untersten sechs Bits der
		//Adresse hochgez�hlt werden (anders als beim Lesen).
		address++;
		eeprom_readWriteByte(*(block+i), (address & 0x0000003f)==0 || i==size-1);
		if((address & 0x0000003f)==0 && i<size-1)  //Adressbits �bergelaufen, aber noch Bytes zu schreiben?
		{
			//Warte bis Schreibvorgang beendet
			do
			{
				eeprom_readWriteByte(0x05, false);  //Befehl "Get Status"
			} while(eeprom_readWriteByte(0x00, true) & 0x01);

			//Neuer "Write Enable"-Befehl
			eeprom_readWriteByte(0x06, true);  //Befehl "Write Enable"
			do
			{
				eeprom_readWriteByte(0x05, false);  //Befehl "Get Status"
			} while((eeprom_readWriteByte(0x00, true) & 0x02)==0x00);  //Warte bis "Write Enable"-Bit gesetzt

			// new "Write" command (mit der n�chsten Adresse)
			eeprom_readWriteByte(0x02, false); // command "Write"
			eeprom_readWriteByte(address >> 8, false);
			eeprom_readWriteByte(address & 0xff, false);
		}
	}

	// wait until write finished
	do
	{
		eeprom_readWriteByte(0x05, false);  // command "Get Status"
	} while(eeprom_readWriteByte(0x00, true) & 0x01);
}

/* read a block from configuration eeprom
 * (address: (0..2047) (block: start address) (size: max. 64)) */
void eeprom_readConfigBlock(u32 address, u8 *block, u32 size)
{
	eeprom_readWriteByte(0x03, false);  // command "Read"
	eeprom_readWriteByte(address >> 8, false);
	eeprom_readWriteByte(address & 0xff, false);

	u32 i;
	for(i=0; i<size; i++)
		*(block+i) = eeprom_readWriteByte(0, i==size-1);  //beim letzten Byte EEPROM deselektieren
}

