/*
 * UART.c
 *
 *  Created on: 25.04.2014
 *      Author: ok/ed
 */

#include "UART.h"

static const u32 baudrateTable[] = {9600, 14400, 19200, 28800, 38400, 57600, 76800, 115200};

#define UART_BUFFER_SIZE 	32
#define UART_TIMEOUT_VALUE 	5

// variables for UART0
static volatile int UART_RxWritePtr;
static volatile int UART_RxReadPtr;

static volatile int UART_TxWritePtr;
static volatile int UART_TxReadPtr;

static volatile char UART_TxBuffer[UART_BUFFER_SIZE];
static volatile char UART_RxBuffer[UART_BUFFER_SIZE];

static volatile u32 UARTTransmitDelay;
volatile u8 UARTTimeoutFlag;
volatile u32 UARTTimeoutTimer;
volatile u32 UARTTransmitDelayTimer;

#if defined(TMCM_USE_UART_0)
#define UART_TMCL_BASE_PTR UART0_BASE_PTR
#elif defined(TMCM_USE_UART_1)
#define UART_TMCL_BASE_PTR UART1_BASE_PTR
#endif

#if defined(TMC_USE_UART_0)
#define UART_TMC_BASE_PTR UART0_BASE_PTR
#elif defined(TMC_USE_UART_1)
#define UART_TMC_BASE_PTR UART1_BASE_PTR
#endif

void uart_init(const UART_MemMapPtr uartch, int sysclk, u8 baudrateIndex)
{
	unsigned short divisor;

	// enable the clock to the selected UART
	if(uartch == UART0_BASE_PTR)
	{
		SIM_SCGC|=SIM_SCGC_UART0_MASK;
	}
	else if (uartch == UART1_BASE_PTR)
	{
		SIM_SCGC|=SIM_SCGC_UART1_MASK;
	}
	else if (uartch == UART2_BASE_PTR)
	{
		SIM_SCGC|=SIM_SCGC_UART2_MASK;
	}

	UART_C2_REG(uartch) = 0;
	UART_C1_REG(uartch) = 0;
	UART_C3_REG(uartch) = 0;
	UART_S2_REG(uartch) = 0;

	// Make sure that the transmitter and receiver are disabled while we
	// change settings.
//        UART_C2_REG(uartch) &= ~(UART_C2_TE_MASK | UART_C2_RE_MASK);

    // Configure the UART for 8-bit mode, no parity
    //UART_C1_REG(uartch) = 0;

	if(baudrateIndex > 7)
		baudrateIndex = 0;  // if baudrate not supported, set to default: 9600

	divisor = (((sysclk)>>4) + (baudrateTable[baudrateIndex]>>1)) / baudrateTable[baudrateIndex];
	UART_BDH_REG(uartch) = (UART_BDH_REG(uartch) & ~(UART_BDH_SBR_MASK)) |  UART_BDH_SBR(divisor >> 8);
	UART_BDL_REG(uartch) = (divisor & UART_BDL_SBR_MASK);

	// enable receiver and transmitter + their interrupts
	UART_C2_REG(uartch) |= (UART_C2_TE_MASK | UART_C2_RE_MASK | UART_C2_RIE_MASK);

	if(uartch == UART0_BASE_PTR)
	{
		enable_irq(INT_UART0-16);
	}
	else if (uartch == UART1_BASE_PTR)
	{
		enable_irq(INT_UART1-16);
	}
	else if (uartch == UART2_BASE_PTR)
	{
		enable_irq(INT_UART2-16);
	}
}
//
//void uart_suppressTx(UART_MemMapPtr uartch)
//{
//	// disable transmitter
//	UART_C2_REG(uartch) &= ~UART_C2_TE_MASK;
//}

void uart_tmcl_init(const u8 baudrateIndex)
{
	uart_init(UART_TMCL_BASE_PTR, BUS_CLOCK, baudrateIndex);
}

//void uart_tmcl_rxInit(u8 baudrateIndex)
//{
//	uart_init(UART_TMCL_BASE_PTR, BUS_CLOCK, baudrateIndex);
//	uart_suppressTx(UART_TMCL_BASE_PTR);
//}
//
//void uart_deinit(UART_MemMapPtr uartch)
//{
//	if(uartch == UART0_BASE_PTR)
//	{
//		disable_irq(INT_UART0-16);
//	}
//	else if (uartch == UART1_BASE_PTR)
//	{
//		disable_irq(INT_UART1-16);
//	}
//	else if (uartch == UART2_BASE_PTR)
//	{
//		disable_irq(INT_UART2-16);
//	}
//
//	// enable receiver and transmitter + their interrupts
//	UART_C2_REG(uartch) &= ~(UART_C2_TE_MASK | UART_C2_RE_MASK | UART_C2_RIE_MASK);
//
//	// disable the clock to the selected UART
//	if(uartch == UART0_BASE_PTR)
//	{
//		SIM_SCGC &= ~SIM_SCGC_UART0_MASK;
//	}
//	else if (uartch == UART1_BASE_PTR)
//	{
//		SIM_SCGC &= ~SIM_SCGC_UART1_MASK;
//	}
//	else if (uartch == UART2_BASE_PTR)
//	{
//		SIM_SCGC &= ~SIM_SCGC_UART2_MASK;
//	}
//}
//
//void uart_tmcl_deinit()
//{
//	uart_deinit(UART_TMCL_BASE_PTR);
//}


void uart_tmcl_write(char ch)
{
	// move char to buffer
	int i = UART_TxWritePtr+1;
	if(i == UART_BUFFER_SIZE)
		i = 0;

	if(i != UART_TxReadPtr)
	{
		UART_TxBuffer[UART_TxWritePtr] = ch;
		UART_TxWritePtr = i;

		// enable send interrupt
		UART_C2_REG(UART_TMCL_BASE_PTR) |= UART_C2_TIE_MASK;
	}
}



bool uart_tmcl_read(char *ch)
{
	// no char available?
	if(UART_RxReadPtr == UART_RxWritePtr)
		return FALSE;

	// get char from buffer
	*ch = UART_RxBuffer[UART_RxReadPtr++];

	if(UART_RxReadPtr == UART_BUFFER_SIZE)
		UART_RxReadPtr = 0;

	return TRUE;
}

bool uart_tmcl_checkTimeout()
{
	if(UARTTimeoutFlag)
	{
		UARTTimeoutFlag = FALSE;
		return TRUE;
	}
	return FALSE;
}

void uart_tmcl_setTransmitDelay(u32 delay)
{
	UARTTransmitDelay = delay;
}

//void UART0_IRQHandler(void)
//{
//    u32 status = UART0_S1;
//
//    if(status & UART_S1_RDRF_MASK)
//    {
//    	// copy char to receive buffer
//    	int i = UART_RxWritePtr+1;
//    	if(i == UART_BUFFER_SIZE)
//    		i = 0;
//
//    	if(i != UART_RxReadPtr)
//    	{
//    		UART_RxBuffer[UART_RxWritePtr] = UART0_D;
//    		UART_RxWritePtr = i;
//    	}
//
//    	// reset timeout value
//    	UARTTimeoutTimer = UART_TIMEOUT_VALUE;
//
//		// Sendeverz�gerung auf Startwert setzen
//		UARTTransmitDelayTimer = UARTTransmitDelay;
//    }
//
//	if(status & UART_S1_TC_MASK)
//	{
//		//letztes Bit gesendet => RS485 wieder auf Empfangen schalten
//
//		UART0_C2 &= ~UART_C2_TCIE_MASK;
//	}
//
//    if(status & UART_S1_TDRE_MASK)
//    {
//		if(UARTTransmitDelayTimer==0)
//		{
//			if(UART_TxWritePtr != UART_TxReadPtr)
//			{
//				// if character in buffer -> copy character to send register
//				UART0_D = UART_TxBuffer[UART_TxReadPtr++];
//
//				if(UART_TxReadPtr == UART_BUFFER_SIZE)
//					UART_TxReadPtr = 0;
//
//				// Transmission-Complete-Interrupt f�r RS485-Richtungsumschaltung aktivieren
//				UART0_C2 |= UART_C2_TCIE_MASK;
//			}
//			else
//			{
//				// empty buffer -> turn off send interrupt
//				UART0_C2 &= ~UART_C2_TIE_MASK;
//			}
//		}
//    }
//}

__attribute__((__used__))
void UART1_IRQHandler(void)
{
    u32 status = UART1_S1;

    if(status & UART_S1_RDRF_MASK)
    {
    	// copy char to receive buffer
    	int i = UART_RxWritePtr+1;
    	if(i == UART_BUFFER_SIZE)
    		i = 0;

    	if(i != UART_RxReadPtr)
    	{
    		UART_RxBuffer[UART_RxWritePtr] = UART1_D;
    		UART_RxWritePtr = i;
    	}

    	// reset timeout value
    	UARTTimeoutTimer = UART_TIMEOUT_VALUE;

		// Sendeverz�gerung auf Startwert setzen
		UARTTransmitDelayTimer = UARTTransmitDelay;
    }

	if(status & UART_S1_TC_MASK)
	{
		//letztes Bit gesendet => RS485 wieder auf Empfangen schalten

		UART1_C2 &= ~UART_C2_TCIE_MASK;
	}

    if(status & UART_S1_TDRE_MASK)
    {
		if(UARTTransmitDelayTimer==0)
		{
			if(UART_TxWritePtr != UART_TxReadPtr)
			{
				// if character in buffer -> copy character to send register
				UART1_D = UART_TxBuffer[UART_TxReadPtr++];

				if(UART_TxReadPtr == UART_BUFFER_SIZE)
					UART_TxReadPtr = 0;

				// Transmission-Complete-Interrupt f�r RS485-Richtungsumschaltung aktivieren
				UART1_C2 |= UART_C2_TCIE_MASK;
			}
			else
			{
				// empty buffer -> turn off send interrupt
				UART1_C2 &= ~UART_C2_TIE_MASK;
			}
		}
    }
}

void tmc_uart_init(){
	unsigned short divisor;

	// enable the clock to the selected UART
	if(UART_TMC_BASE_PTR == UART0_BASE_PTR)
	{
		SIM_SCGC|=SIM_SCGC_UART0_MASK;
	}
	else if (UART_TMC_BASE_PTR == UART1_BASE_PTR)
	{
		SIM_SCGC|=SIM_SCGC_UART1_MASK;

	}
	else if (UART_TMC_BASE_PTR == UART2_BASE_PTR)
	{
		SIM_SCGC|=SIM_SCGC_UART2_MASK;

	}
		UART_C2_REG(UART_TMC_BASE_PTR) = 0;
		UART_C1_REG(UART_TMC_BASE_PTR) = 0;
		UART_C3_REG(UART_TMC_BASE_PTR) = 0;
		UART_S2_REG(UART_TMC_BASE_PTR) = 0;



	divisor = (((BUS_CLOCK)>>4) + (115200>>1)) / 115200;
	UART_BDH_REG(UART_TMC_BASE_PTR) = (UART_BDH_REG(UART_TMC_BASE_PTR) & ~(UART_BDH_SBR_MASK)) |  UART_BDH_SBR(divisor >> 8);
	UART_BDL_REG(UART_TMC_BASE_PTR) = (divisor & UART_BDL_SBR_MASK);

	// enable receiver and transmitter + their interrupts
	UART_C2_REG(UART_TMC_BASE_PTR) |= (UART_C2_TE_MASK | UART_C2_RE_MASK);
}

void tmc_uart_write(uint8_t c){
	while (!(UART_S1_REG(UART_TMC_BASE_PTR) & UART_S1_TDRE_MASK))
	{

	}
	UART_D_REG(UART_TMC_BASE_PTR) = c;
}

uint8_t tmc_uart_read(){
	while (!(UART_S1_REG(UART_TMC_BASE_PTR) & UART_S1_RDRF_MASK))
	{

	}

	return UART_D_REG(UART_TMC_BASE_PTR);
}

void tmc_uart_readWriteArray(uint8_t channel, uint8_t *data, size_t writeLength, size_t readLength){
	UNUSED(channel);

	for (size_t i = 0; i < writeLength; i++){
		tmc_uart_write(data[i]);
	}

	if(readLength){
		// wait for transmission to finish
		while(!(UART_S1_REG(UART_TMC_BASE_PTR) & UART_S1_TC_MASK)){

		}

		for(uint32_t i = 0; i < 100; i++){
			asm volatile ("nop\n\t");
		}

		// clear rx buffer and errors
		while(UART_S1_REG(UART_TMC_BASE_PTR) & UART_S1_RDRF_MASK){
			volatile uint8_t rxchar = UART_D_REG(UART_TMC_BASE_PTR);
		}

		for (size_t i = 0; i < readLength; i++){
			data[i] = tmc_uart_read();
		}
	}
}
