/*
 * TMCM1076_v1.0.c
 *
 *  Created on: 19.03.2018
 *      Author: ok
 */
#include "TMC2300_MOTOR_v2.0.h"

#if DEVICE==TMC2300_MOTOR_V20

#include "../system/SysTick.h"
//#include "../../tmc/commback/Eeprombak.h"
#include "../../tmc/motion/TMC2300.h"
#include "hal/comm/SPI.h"
#include "hal/comm/UART.h"

const char *VersionString="2300V100"; // Update SW_VERSION_HIGH, SW_VERSION_LOW, FIRMWARE_VERSION and TMCM_EEPROM_MAGIC as well



int32_t adcavg_state;
int32_t adcavg;

uint8_t chargeled;

//TMC2300TypeDef tmc2300 = 0;

void tmc2300_readWriteArray(uint8_t channel, uint8_t *data, size_t writeLength, size_t readLength){
	 tmc_uart_readWriteArray(channel, data, writeLength, readLength);

}

uint8_t tmc2300_CRC8(uint8_t *data, size_t length){
	uint8_t crc = 0;
	uint8_t currentByte;

	for (uint8_t i =0; i<(uint8_t)(length); i++) {
	currentByte = data[i];
		for (uint8_t j=0; j<8; j++) {
			if ((crc >> 7) ^ (currentByte&0x01)){
				crc = (crc << 1) ^ 0x07;
			}
			else{
				crc = (crc << 1);
			}
				currentByte = currentByte >> 1;
		}
	}
	return crc;
}

void tmcm_initModuleConfig()
{
	moduleConfig.Baudrate = DEFAULT_SERIAL_BAUD_RATE;				// RS232/RS485 (115200bps)
	moduleConfig.SerialModuleAddress = 1; 							// RS232/RS485;
	moduleConfig.SerialHostAddress = 2;								// Anwortadresse RS232/RS485
	moduleConfig.TelegramPauseTime = 1;								// Pausenzeit RS232/RS485

	moduleConfig.TMCLProtectionMode = 0;				// Schutz f�r TMCL-Programm
	moduleConfig.TMCLAutostartFlag = 0;					// TMCL-Autostart
//	moduleConfig.EEPROMLockFlag = 0;					// EEPROM-Lock
	moduleConfig.ZeroUserVariables = 0;

}

//void tmcm_initMotorConfig()
//{
//	motorConfig.uStepResolution = 4;								// Microstep Resolution (1-9: 1, 1/2, 1/4, ... 1/256)
//	motorConfig.uStepInterpolation = 1;								// Microstep Interpolation (0/1)
//}

void tmcm_initModuleSpecificIO()
{
	// configure input pins
	GPIOA_PIDR &= ~(RXD |
					DIAG |
					TMC_RXD |
					CHGR_STAT |
					CHGR_DISABLE |
					MISO |
					BTN1 |
					BTN2 |
					BTN3);

	// configure output pins
	GPIOA_PDDR |=	TXD |		// LED
					STEP |		// !EN_DRV
					DIR |		// SPI Mode
					EN |		// SD Mode
					TMC_TXD |
					MOSI |
					CS |
					SCK |
					LED1 |
					LED2 |
					LEDRED |
					LEDYELLOW |
					LEDGREEN;		// SPI_CS (Chip Select)

	// set initialization values
	GPIOA_PCOR =	TXD |		// clear LED
					TMC_TXD |		// clear SPI Mode
					MOSI |
					SCK |
					STEP |
					DIR |
					EN;		// clear SD Mode

	GPIOA_PSOR =	LED1 |		// set !EN_DRV
					LED2 |
					LEDRED |
					LEDYELLOW |
					LEDGREEN |
					CS |
					CHGR_DISABLE;		// set SPI_CS

	PORT_PUEL  |=	BTN1 | BTN2 | BTN3;		// Enable  Config pull-ups




	SIM_SCGC|=SIM_SCGC_ADC_MASK | SIM_SCGC_ACMP0_MASK | SIM_SCGC_ACMP1_MASK;

	PMC_SPMSC1 = PMC_SPMSC1_BGBE_MASK; // enable bandgap buffer
	ADC_SC2 = 0;
	ADC_SC3 = ADC_SC3_ADIV(3) | ADC_SC3_ADLSMP_MASK | ADC_SC3_MODE(2) | ADC_SC3_ADICLK(1) ;
	ADC_SC4 = 0;
	ADC_SC1 = ADC_SC1_ADCO_MASK | 0x17; // set continous sample and bandgap channel

	adcavg_state = 0;
	adcavg = 0;
	chargeled = 0;
}

void tmcm_set_led_charge_on()
{
	tmcm_set_led_charge_off();

	switch(chargeled){
	case 0:
		GPIOA_PCOR = LEDRED;
		break;
	case 1:
		GPIOA_PCOR = LEDYELLOW;
		break;
	case 2:
		GPIOA_PCOR = LEDGREEN;
		break;
	}
}

void tmcm_set_led_charge_off()
{
		GPIOA_PSOR = LEDRED | LEDYELLOW | LEDGREEN;
}

void tmcm_determine_led_charge()
{
	switch(chargeled){
	case 0:
		if(tmcm_getModuleVoltage() >= 36)
			chargeled = 1;
		break;
	case 1:
		if(tmcm_getModuleVoltage() <= 35)
			chargeled = 0;
		else if(tmcm_getModuleVoltage() >= 39)
			chargeled = 2;
		break;
	case 2:
		if(tmcm_getModuleVoltage() <= 38)
			chargeled = 1;
		break;
	}
}

bool tmcm_is_charging(){
	return (GPIOA_PDIR & CHGR_STAT)?0:1;
}

void tmcm_handlecharger()
{
	if(ADC_SC1 & ADC_SC1_COCO_MASK){
		adcavg_state += (((int32_t)ADC_R - adcavg) *  (1<<14)) / (1<<14);
		adcavg = adcavg_state / (1<<14);
	}
}

//void tmcm_initModuleSpecificADC()
//{}
//
//void tmcm_initModuleSpecificADCOffsets() {}
//
//void tmcm_triggerAdcMeasurement(u32 delay)
//{
//	UNUSED(delay);
//}

bool tmcm_get_diag_pin()
{
	return (GPIOA_PDIR & DIAG)?1:0;
}

void tmcm_set_step_pin()
{
	GPIOA_PSOR = STEP;
}

void tmcm_clear_step_pin()
{
	GPIOA_PCOR = STEP;
}

void tmcm_set_dir_pin()
{
	GPIOA_PSOR = DIR;
}

void tmcm_clear_dir_pin()
{
	GPIOA_PCOR = DIR;
}

void tmcm_set_SPI_CS()
{
	GPIOA_PSOR = CS;
}

void tmcm_clear_SPI_CS()
{
	GPIOA_PCOR = CS;
}

//void tmcm_set_led_run_on()
//{
//	GPIOA_PCOR = LED1;
//}
//
//void tmcm_set_led_run_off()
//{
//	GPIOA_PSOR = LED1;
//}
//
//void tmcm_toggle_led_run()
//{
//	GPIOA_PDOR ^= LED1;
//}
//
void tmcm_enable_driver()
{
	GPIOA_PSOR = EN;
}

void tmcm_disable_driver()
{
	GPIOA_PCOR = EN;
}

bool tmcm_get_driver_state()
{
	return (GPIOA_PDOR & EN)?1:0;
}

u8 tmcm_getModuleSpecificIOPin(u8 pin)
{
	switch (pin)
	{
	case 0:
		return (GPIOA_PDIR & BTN1)?0:1;
	case 1:
		return (GPIOA_PDIR & BTN2)?0:1;
	case 2:
		return (GPIOA_PDIR & BTN3)?0:1;
	case 3:
		return (GPIOA_PDIR & DIAG)?1:0;
	case 4:
		return (GPIOA_PDIR & CHGR_STAT)?0:1;
	default:
		return 0;
	}
	return 0;
}

u8 tmcm_getModuleSpecificIOPinStatus(u8 pin)
{
	switch (pin)
	{
	case 0:
		return (GPIOA_PDOR & EN)?1:0;
	case 1:
		return (GPIOA_PDOR & LED1)?0:1;
	case 2:
		return (GPIOA_PDOR & LED2)?0:1;
	case 3:
		return (GPIOA_PDDR & CHGR_DISABLE)?1:0;
	default:
		return 0;
	}
}

void tmcm_clearModuleSpecificIOPin(u8 pin)
{
	switch(pin)
	{
	case 0:
		GPIOA_PCOR = EN;
		break;
	case 1:
		GPIOA_PSOR = LED1;
		break;
	case 2:
		GPIOA_PSOR = LED2;
		break;
	case 3:
		GPIOA_PDDR &= ~CHGR_DISABLE;
		break;
	}
}

void tmcm_setModuleSpecificIOPin(u8 pin)
{
	switch(pin)
	{
	case 0:
		GPIOA_PSOR = EN;
		break;
	case 1:
		GPIOA_PCOR = LED1;
		break;
	case 2:
		GPIOA_PCOR = LED2;
		break;
	case 3:
		GPIOA_PDDR |= CHGR_DISABLE;
		break;
	}
}

u32 tmcm_getModuleVoltage(){
	return (4095 * 12) / (adcavg);
}

//u32 tmcm_getModuleSpecificADCValue(u8 pin)
//{
//	UNUSED(pin);
//	return 0;
//}
//
//u32 tmcm_getModuleSpecificADCOffset(u8 pin)
//{
//	UNUSED(pin);
//	return 0;
//}
//
//void tmcm_setModuleSpecificADCOffset(u8 pin, u16 offset)
//{
//	UNUSED(pin);
//	UNUSED(offset);
//}

void tmcm_checkHardwareReset()
{
	//Test if SWCLK and SWDIO are connected together
	//If so: Reset to default settings

	// SWD_DIO on PTA4 and SWD_CLK on PTC4
	SIM_SOPT &= ~SIM_SOPT_SWDE_MASK;
	GPIOA_PDDR |= PTA4;			// SWD_DIO (output)
	GPIOA_PIDR &= ~PTC4;		// SWD_CLK (input)

	bool reset = 1;

	for(u32 i=0; i<255; i++)
	{
		GPIOA_PCOR = PTA4;
		for(u32 j=0; j<255; j++)
			__asm__("nop");
		if((GPIOA_PDIR & PTC4))
			reset = 0;

		GPIOA_PSOR = PTA4;
		for(u32 j=0; j<255; j++)
			__asm__("nop");
		if(!(GPIOA_PDIR & PTC4))
			reset = 0;
	}

	ResetToFactoryDefaultRequested=reset;

	//Configure both pints again to SWCLK/SWDIO to make the module programmable again
	GPIOA_PDDR &= ~PTA4;		// SWD_DIO (reset pin direction)
	GPIOA_PIDR |= PTC4;			// SWD_CLK (reset pin direction)
	SIM_SOPT |= SIM_SOPT_SWDE_MASK;
}

//void tmcm_checkResetToDefault()
//{
//	volatile u32 i;
//
//	if(ResetToFactoryDefaultRequested)
//	{
//		tmcm_initModuleConfig();
////		tmcm_initMotorConfig();
//
//		// initialize EEPROM support
//		eeprom_initSpi();
//		eeprom_initConfig();
//		force_bootloader();
//	}
//}



#endif
