/*
 * TMCM1076_v1.0.h
 *
 *  Created on: 19.03.2018
 *      Author: ok
 */

#ifndef TMC2300_MOTOR_V20_H_
#define TMC2300_MOTOR_V20_H_

#include "SelectModule.h"

#if DEVICE==TMC2300_MOTOR_V20

	#include "../../Definitions.h"

	#include "../cpu/MKE02Z64xxx4/nvic.h"
	#include "../cpu/MKE02Z64xxx4/MKE02Z5_IOPins.h"
	#include "../cpu/MKE02Z64xxx4/PDD/GPIO_PDD.h"

	/* Including needed modules to compile this module/procedure */
	#include "../cpu/MKE02Z64xxx4/KE-Cpu.h"
	#include "hal/cpu/MKE02Z64xxx4/Events.h"
	#include "hal/cpu/MKE02Z64xxx4/PE/Bit1.h"

	/* Including shared modules, which are used for whole project */
	#include "hal/cpu/MKE02Z64xxx4/PE/PE_Types.h"
	#include "hal/cpu/MKE02Z64xxx4/PE/PE_Error.h"
	#include "hal/cpu/MKE02Z64xxx4/PE/PE_Const.h"
	#include "hal/cpu/MKE02Z64xxx4/PE/IO_Map.h"

	#define BOARD_CPU MKE02Z164VFM4

	#include "../../tmc/Ports.h"
	#include "../../tmc/motion/TMC2300.h"
	#include "../../tmc/motion/helpers/Config.h"

	#define TMCM_USE_ALIVE_LED
	#define TMCM_USE_UART_INTERFACE
	//#define TMCM_USE_UART_0
	#define TMCM_USE_UART_1
	//#define TMCM_USE_SPI_1

	#define DEVTYPE_TMC2300
	#define MOTOR_2300 0
	#define TMC_USE_UART_0


//	#define MODE5130_NONE	0
//	#define MODE5130_SPI	1
//	#define MODE5130_SD		2
//	#define MODE5130_BOTH	3

	#define CORE_CLOCK ((u32)32000000uL) // 32 MHz Core Clock
	#define BUS_CLOCK ((u32)16000000uL) // 16 MHz Bus Clock (Core Clock divided by 2 (BDIV))

	#define USE_EXTENDED_PARAMETER_LIST FALSE
	#define USE_VEL_FACTOR				FALSE
	#define USE_MOTION_PARAMETERS		FALSE

#if USE_MOTION_PARAMETERS == FALSE
	#define USE_VEL_FACTOR				FALSE // in this case always false
#endif

	// for documentation only
	#define MODULE_NAME				TMCM-2300
	#define MODULE_FW_VERSION_NAME	2300
	#define HARDWARE_VERSION		2.0
	#define FIRMWARE_VERSION		1.00
//	#define s16_MAX					(s16)32767
//	#define s16_MIN    				(s16)-32768
//	#define u16_MAX					(u16)65535
//	#define s32_MAX    				(s32)2147483647
//	#define s32_MIN    				(s32)-2147483648
//	#define u32_MAX					(u32)4294967295uL

	#define MAX_VELOCITY 			(s32)7999774
	#define MAX_ACCELERATION		(s32)7629278
	#define DEFAULT_SERIAL_BAUD_RATE	(u8)7

	// module number in HEX (1076)
	#define SW_TYPE_HIGH 0x08
	#define SW_TYPE_LOW  0xFC

	#define USE_TEMPERATURE_MEASUREMENT
	#define MIN_CRITICAL_TEMP     100  //100°C
	#define MIN_SWITCHON_TEMP     110  //110°C
	#define MAX_CRITICAL_TEMP     120  //120°C


	// UARTS
	#define TXD PTC7
	#define RXD PTC6

	// TMC
	#define STEP PTC1
	#define DIR PTC0
	#define EN PTC2
	#define DIAG PTC3
	#define TMC_TXD PTB1
	#define TMC_RXD PTB0

	// CHARGER
	#define CHGR_DISABLE PTA7
	#define CHGR_STAT PTA6

	// INPUT
	#define BTN1 PTD3
	#define BTN2 PTD2
	#define BTN3 PTA0

	// LED
	#define LED1 PTD1
	#define LED2 PTD0
	#define LEDRED PTA1
	#define LEDYELLOW PTA2
	#define LEDGREEN PTA3

	// SPI
	#define MISO PTB4
	#define MOSI PTB3
	#define SCK PTB2
	#define CS PTB5

	// digital inputs
	#define TMCM_USE_DIGITAL_INPUT_0	TRUE
	#define TMCM_DIGITAL_INPUT_0_NAME	CHOP

	#define TMCM_USE_DIGITAL_INPUT_1	TRUE
	#define TMCM_DIGITAL_INPUT_1_NAME	Enable

#if USE_MOTION_PARAMETERS == FALSE

	#define TMCM_USE_DIGITAL_INPUT_2	FALSE
	#define TMCM_USE_DIGITAL_INPUT_3	FALSE
	#define TMCM_USE_DIGITAL_INPUT_4	FALSE
	#define TMCM_USE_DIGITAL_INPUT_5	FALSE
	#define TMCM_USE_DIGITAL_INPUT_6	FALSE
	#define TMCM_USE_DIGITAL_INPUT_7	FALSE

	// digital outputs
	#define TMCM_USE_DIGITAL_OUTPUT_0	FALSE
	#define TMCM_USE_DIGITAL_OUTPUT_1	FALSE
	#define TMCM_USE_DIGITAL_OUTPUT_2	FALSE
#else
	#define TMCM_USE_DIGITAL_INPUT_2	TRUE
	#define TMCM_DIGITAL_INPUT_2_NAME	XConf

	#define TMCM_USE_DIGITAL_INPUT_3	TRUE
	#define TMCM_DIGITAL_INPUT_3_NAME	uStepConf1

	#define TMCM_USE_DIGITAL_INPUT_4	TRUE
	#define TMCM_DIGITAL_INPUT_4_NAME	uStepConf2

	#define TMCM_USE_DIGITAL_INPUT_5	TRUE
	#define TMCM_DIGITAL_INPUT_5_NAME	CurSet1

	#define TMCM_USE_DIGITAL_INPUT_6	TRUE
	#define TMCM_DIGITAL_INPUT_6_NAME	CurSet2

	#define TMCM_USE_DIGITAL_INPUT_7	TRUE
	#define TMCM_DIGITAL_INPUT_7_NAME	CurSet4

	// digital outputs
	#define TMCM_USE_DIGITAL_OUTPUT_0	TRUE
	#define TMCM_DIGITAL_OUTPUT_0_NAME	EnableDriver

	#define TMCM_USE_DIGITAL_OUTPUT_1	TRUE
	#define TMCM_DIGITAL_OUTPUT_1_NAME	SPIMode

	#define TMCM_USE_DIGITAL_OUTPUT_2	TRUE
	#define TMCM_DIGITAL_OUTPUT_2_NAME	SDMode
#endif

	TModuleConfig moduleConfig;
//	TMotorConfig motorConfig;

	TMC2300TypeDef tmc2300;

	ConfigurationTypeDef tmc2300_config;

#endif

#endif /* TMC1076_V10_H_ */
