
#include "Definitions.h"
#include "hal/system/SysTick.h"
#include "hal/system/Cpu.h"
#include "hal/comm/SPI.h"
#include "TMCL.h"
#include "StepDir.h"
#include "hal/comm/UART.h"

__attribute__((__used__)) __attribute__ ((section(".bldata"))) u32 BLMagic;

void tmcm_doSystemCheck()
{
#if defined(TMCM_USE_ALIVE_LED)
	static u32 lastTime = 0;
	static s32 state = 0;
	u32 actualTime = systick_getTimer();

	if ((actualTime-lastTime) > 500)
	{
		if (state == 0)
		{
			if(tmcm_is_charging())
				tmcm_set_led_charge_off();

			tmcm_determine_led_charge();

			state = 1;
		} else {
			tmcm_set_led_charge_on();
			state = 0;
		}
		lastTime = actualTime;
	}
#endif
}

int main(void)
{
	cpu_init();
	systick_init();

	// initialize configuration settings
	tmcm_initModuleConfig();
	//tmcm_initMotorConfig();

	// initialize hardware
	tmcm_initModuleSpecificIO();

	// check reset to factory default
	tmcm_checkHardwareReset();

	// initialize EEPROM support
	eeprom_initSpi();
	eeprom_initConfig();

	// initialize TMC2300
	tmcm_disable_driver();
	tmc_uart_init();
	tmc2300_init(&tmc2300, 0, &tmc2300_config, &tmc2300_defaultRegisterResetState[0]);
	tmc2300_restore(&tmc2300);

	while(tmc2300.config->state != CONFIG_READY)
	{
		tmc2300_periodicJob(&tmc2300, 0);
	}

	TMC2300_FIELD_UPDATE(&tmc2300, TMC2300_IHOLD_IRUN, TMC2300_IRUN_MASK, TMC2300_IRUN_SHIFT, 16);


	// initialize Step Direction interface
	StepDir_init(STEPDIR_FREQUENCY);

	// initialize tmcl communication
	uart_tmcl_init(moduleConfig.Baudrate);

	// initialize software
	tmcl_init();

	for(;;)
	{
		tmcm_doSystemCheck();

		tmcl_processCommand(TRUE);

		tmcm_handlecharger();

		StepDir_periodicJob(0);
	}
}
