/*
 * TMCL-Compute.c
 *
 *  Created on: 16.04.2012
 *      Author: ed
 */

#include "TMCL-Compute.h"
#include "StepDir.h"
#include "hal/comm/SPI.h"
#include "hal/system/SysTick.h"

//extern uint32_t tmcl_handleAxisParameter(uint8_t motor, uint8_t command, uint8_t type, int32_t *value);
//extern void tmcl_handleGlobalParameter(u8 command);

/**********************************
   Funktion: AccuToAxisParameter()

   Zweck: TMCL-Befehl AAP
 *********************************/
void AccuToAxisParameter()
{
	ActualCommand.Value.Int32 = TMCLAccu;
	tmcl_handleAxisParameter(ActualCommand.Motor, TMCL_SAP, ActualCommand.Type,  (int32_t *)&TMCLAccu); // set axis parameter
}

/************************************
   Funktion: AccuToCoordinate()

   Zweck: TMCL-Befehl ACO
 ************************************/
void AccuToCoordinate()
{
	if(ActualCommand.Motor == 0)
	{
		if(ActualCommand.Type<TMCL_COORDINATES)
		{
			TMCLCoordinate[ActualCommand.Type] = TMCLAccu;
		}
		else ActualReply.Status = REPLY_WRONG_TYPE;
	}
	else ActualReply.Status = REPLY_INVALID_VALUE;
}

/************************************
   Funktion: AccuToGlobalParameter()

   Zweck: TMCL-Befehl AGP
 ************************************/
void AccuToGlobalParameter(void)
{
	ActualCommand.Value.Int32=TMCLAccu;
	tmcl_handleGlobalParameter(TMCL_SGP);
}

/*********************************
   Funktion: ApplStop()

   Zweck: TMCL-Befehl 129
          (run application)
 *********************************/
void ApplRun()
{
	TMCLMode = TM_RUN;
	if(ActualCommand.Type==1)
		TMCLProgramCounter=ActualCommand.Value.Int32;
}

/*********************************
   Funktion: ApplStep()

   Zweck: TMCL-Befehl 130
          (step application)
 *********************************/
void ApplStep()
{
	TMCLMode = TM_STEP;
}

/*********************************
   Funktion: ApplReset()

   Zweck: TMCL-Befehl 131
          (reset application)
 *********************************/
void ApplReset()
{
//#if N_O_TMCL_INTERRUPTS > 0
//	u32 i;

//	GlobalInterruptsEnabled=FALSE;
//	GlobalInterruptFlag=FALSE;
//	for(i=0; i<N_O_TMCL_INTERRUPTS; i++)
//	{
//		InterruptVector[i]=TMCL_INTERRUPT_NOT_ASSIGNED;
//		InterruptFlag[i]=TMCL_INTERRUPT_DISABLED;
//	}
//#endif

	TMCLMode = TM_IDLE;
	TMCLProgramCounter = 0;
	TMCLStackPointer = 0;
	TMCLWaitFlag = 0;
	TMCLAccu = 0;
	TMCLXReg = 0;
	TMCLCPUFlags = 0;
}

/*********************************
   Funktion: ApplStop()

   Zweck: TMCL-Befehl 128
          (stop application)
 *********************************/
void ApplStop(void)
{
	TMCLMode=TM_IDLE;
}

/*********************************
   Funktion: Breakpoint()

   Zweck: TMCL-Befehl 141
          (set breakpoint)
 *********************************/
void Breakpoint()
{
	u32 i, j;

	switch(ActualCommand.Type)
	{
    	case 0:  //Breakpoint hinzuf�gen
    		if(TMCLBreakpointCount<TMCL_BREAKPOINTS)
    			TMCLBreakpoint[TMCLBreakpointCount++]=(u32) ActualCommand.Value.Int32;
    		else
    			ActualReply.Status=REPLY_MAX_EXCEEDED;
    		break;
    	case 1:  //Breakpoint l�schen
    		for(i=0; i<TMCLBreakpointCount; i++)
    		{
    			if(TMCLBreakpoint[i]==(u32) ActualCommand.Value.Int32)
    			{
    				for(j=i; j<TMCLBreakpointCount-1; j++)
    					TMCLBreakpoint[j]=TMCLBreakpoint[j+1];
    				TMCLBreakpointCount--;
    				break;
    			}
    		}
    		break;
    	case 2:  //Alle Breakpoints l�schen
    		TMCLBreakpointCount=0;
    		for(i=0; i<TMCL_BREAKPOINTS; i++)
    			TMCLBreakpoint[i]=0;
    		break;
    	case 3:  //Maximale Anzahl von Breakpoints ermitteln
    		ActualReply.Value.Int32=TMCL_BREAKPOINTS;
    		break;
    	default:
    		ActualReply.Status=REPLY_WRONG_TYPE;
    		break;
	}
}

/*********************************
   Funktion: Calculate()

   Zweck: TMCL-Befehl CALC
 *********************************/
void Calculate()
{
	switch(ActualCommand.Type)
	{
    	case CALC_ADD:
    		TMCLAccu += ActualCommand.Value.Int32;
    		break;
    	case CALC_SUB:
    		TMCLAccu -= ActualCommand.Value.Int32;
    		break;
    	case CALC_MUL:
    		TMCLAccu *= ActualCommand.Value.Int32;
    		break;
    	case CALC_DIV:
    		TMCLAccu /= ActualCommand.Value.Int32;
    		break;
    	case CALC_MOD:
    		TMCLAccu %= ActualCommand.Value.Int32;
    		break;
    	case CALC_AND:
    		TMCLAccu &= ActualCommand.Value.Int32;
    		break;
    	case CALC_OR:
    		TMCLAccu |= ActualCommand.Value.Int32;
    		break;
    	case CALC_XOR:
    		TMCLAccu ^= ActualCommand.Value.Int32;
    		break;
    	case CALC_NOT:
    		TMCLAccu =~TMCLAccu;
    		break;
    	case CALC_LOAD:
    		TMCLAccu = ActualCommand.Value.Int32;
    		break;
	}

	if(TMCLAccu==0)
		TMCLCPUFlags|=FLAG_ZERO;
	else
		TMCLCPUFlags&=~FLAG_ZERO;
}

/**********************************
   Funktion: CalculateXRegister()

   Zweck: TMCL-Befehl CALCX
 **********************************/
void CalculateXRegister()
{
	u32 Swap;

	switch(ActualCommand.Type)
	{
    	case CALC_ADD:
    		TMCLAccu += TMCLXReg;
    		break;
    	case CALC_SUB:
    		TMCLAccu -= TMCLXReg;
    		break;
    	case CALC_MUL:
    		TMCLAccu *= TMCLXReg;
    		break;
    	case CALC_DIV:
    		TMCLAccu /= TMCLXReg;
    		break;
    	case CALC_MOD:
    		TMCLAccu %= TMCLXReg;
    		break;
    	case CALC_AND:
    		TMCLAccu &= TMCLXReg;
    		break;
    	case CALC_OR:
    		TMCLAccu |= TMCLXReg;
    		break;
    	case CALC_XOR:
    		TMCLAccu ^= TMCLXReg;
    		break;
    	case CALC_NOT:
    		TMCLXReg = ~TMCLXReg;
    		break;
    	case CALC_LOAD:
    		TMCLXReg = TMCLAccu;
    		break;
    	case CALC_SWAP:
    		Swap = TMCLAccu;
    		TMCLAccu = TMCLXReg;
    		TMCLXReg = Swap;
    		break;
	}

	if(TMCLAccu == 0)
		TMCLCPUFlags |= FLAG_ZERO;
	else
		TMCLCPUFlags &= ~FLAG_ZERO;
}

/*********************************
   Funktion: ClearError()

   Zweck: TMCL-Befehl CLE
 *********************************/
void ClearError()
{
	switch(ActualCommand.Type)
	{
    	case CLE_ALL:
    		TMCLCPUFlags &= 0x0000000f;
    		break;
    	case CLE_ETO:
    		TMCLCPUFlags &= ~FLAG_ERROR_TIMEOUT;
    		break;
    	case CLE_EAL:
    		TMCLCPUFlags &= ~FLAG_ERROR_EXT_ALARM;
    		break;
    	case CLE_EDV:
    		TMCLCPUFlags &= ~FLAG_ERROR_DEVIATION;
    		break;
    	case CLE_EPO:
    		TMCLCPUFlags &= ~FLAG_ERROR_POSITION;
    		break;
    	case CLE_ESD:
    		TMCLCPUFlags &= ~FLAG_ERROR_SHUTDOWN;
    		break;
	}
}

/*********************************
   Funktion: Compare()

   Zweck: TMCL-Befehl COMP
 *********************************/
void Compare()
{
	if(TMCLAccu>ActualCommand.Value.Int32)
	{
		TMCLCPUFlags |= FLAG_GREATER;
		TMCLCPUFlags &= ~FLAG_LOWER;
		TMCLCPUFlags &= ~FLAG_EQUAL;
	}
	else if(TMCLAccu == ActualCommand.Value.Int32)
	{
		TMCLCPUFlags &= ~FLAG_GREATER;
		TMCLCPUFlags &= ~FLAG_LOWER;
		TMCLCPUFlags |= FLAG_EQUAL;
	}
	else
	{
		TMCLCPUFlags &= ~FLAG_GREATER;
		TMCLCPUFlags |= FLAG_LOWER;
		TMCLCPUFlags &= ~FLAG_EQUAL;
	}

	if(TMCLAccu==0)
		TMCLCPUFlags |= FLAG_ZERO;
	else
		TMCLCPUFlags &= ~FLAG_ZERO;
}

/* enter download mode (TMCL command 132) */
void DownloadStart(u8 TMCLProtectionMode/*, u32 statusFlags*/)
{
	if(!(TMCLProtectionMode & PB_WRITE))
	{
		// allow download mode only if motor is halted and not in positioning / or velocity mode
//		if ((statusFlags & MOTORHALTED) || (statusFlags & STOP_MODE))
//		{
			TMCLMode = TM_DOWNLOAD;
			TMCLMemoryPointer = ActualCommand.Value.Int32;
//		} else {
//			ActualReply.Status = REPLY_DOWNLOAD_NOT_POSSIBLE;
//		}
	} else {
		ActualReply.Status = REPLY_WRITE_PROTECTED;
	}
}

/* exit download mode (TMCL command 123) */
void DownloadEnd()
{
	TMCLMode = TM_IDLE;
}

/****************************************
   Funktion: FactoryDefault()

   Zweck: Zur�cksetzen auf
          Werkseinstellungen.
 ***************************************/
void FactoryDefault()
{
//	if(ActualCommand.Type==0 && ActualCommand.Motor==0 && ActualCommand.Value.Int32==1234)
//	{
//		eeprom_writeConfigByte(TMCM_ADDR_EEPROM_MAGIC, 0);
//		eeprom_eraseTMCLProgram();
//		cpu_reset(true);
//	}
}

/**********************************
   Funktion: GetApplStatus()

   Zweck: TMCL-Befehl 135
          (get application status)
 **********************************/
void GetApplStatus()
{
	switch(ActualCommand.Type)
	{
    	case 0:
    		ActualReply.Value.Byte[3] = TMCLMode;
    		ActualReply.Value.Byte[2] = TMCLWaitFlag;
    		ActualReply.Value.Byte[1] = TMCLMemoryPointer >> 8;
    		ActualReply.Value.Byte[0] = TMCLMemoryPointer;
    		break;
    	case 1:
    		ActualReply.Value.Byte[3] = TMCLMode;
    		ActualReply.Value.Byte[2] = TMCLWaitFlag;
    		ActualReply.Value.Byte[1] = TMCLProgramCounter >> 8;
    		ActualReply.Value.Byte[0] = TMCLProgramCounter;
    		break;
    	case 2:
    		ActualReply.Value.Int32 = TMCLAccu;
    		break;
    	case 3:
    		ActualReply.Value.Int32 = TMCLXReg;
    		break;
	}
}

void IgnoreCommand()
{
	//Spezialfall: dieser Befehl wird v�llig ignoriert => keine Antwort
	TMCLReplyFormat = RF_NO_REPLY;
}

/*********************************
   Funktion: JumpAlways()

   Zweck: TMCL-Befehl JA
 *********************************/
void JumpAlways()
{
	TMCLProgramCounter = ActualCommand.Value.Int32;
}

/*********************************
   Funktion: JumpConditional()

   Zweck: TMCL-Befehl JC
 *********************************/
void JumpConditional(void)
{
	switch(ActualCommand.Type)
	{
    	case JC_ZE:
    		if(TMCLCPUFlags & FLAG_ZERO)
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_NZ:
    		if(!(TMCLCPUFlags & FLAG_ZERO))
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_EQ:
    		if((TMCLCPUFlags & FLAG_EQUAL))
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_NE:
    		if(!(TMCLCPUFlags & FLAG_EQUAL))
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_GT:
    		if((TMCLCPUFlags & FLAG_GREATER))
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_GE:
    		if((TMCLCPUFlags & FLAG_GREATER) || (TMCLCPUFlags & FLAG_EQUAL))
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_LT:
    		if((TMCLCPUFlags & FLAG_LOWER))
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_LE:
    		if((TMCLCPUFlags & FLAG_LOWER) || (TMCLCPUFlags & FLAG_EQUAL))
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_ETO:
    		if(TMCLCPUFlags & FLAG_ERROR_TIMEOUT)
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_EAL:
    		if(TMCLCPUFlags & FLAG_ERROR_EXT_ALARM)
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_EDV:
    		if(TMCLCPUFlags & FLAG_ERROR_DEVIATION)
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_EPO:
    		if(TMCLCPUFlags & FLAG_ERROR_POSITION)
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
    	case JC_ESD:
    		if(TMCLCPUFlags & FLAG_ERROR_SHUTDOWN)
    			TMCLProgramCounter = ActualCommand.Value.Int32;
    		break;
	}
}

/*********************************
   Funktion: CallSubroutine()

   Zweck: TMCL-Befehl CSUB
 *********************************/
void CallSubroutine()
{
	if(TMCLStackPointer<TMCL_STACK_DEPTH)
	{
		TMCLStack[TMCLStackPointer++] = TMCLProgramCounter;
		TMCLProgramCounter = ActualCommand.Value.Int32;
	}
}

/*********************************
   Funktion: ReadTMCLMemory()

   Zweck: TMCL-Befehl 134
          (read TMCL memory)
 *********************************/
void ReadTMCLMemory(u8 SerialHostAddress, u8 TMCLProtectionMode)
{
	u32 i;
	TTMCLCommand TMCLMemory;

	TMCLReplyFormat = RF_SPECIAL;
	SpecialReply[0] = SerialHostAddress;
	if(ActualCommand.Value.Int32<=TMCL_MEM_SIZE && !(TMCLProtectionMode & PB_READ))
	{
		eeprom_readTMCLCommand(ActualCommand.Value.Int32, &TMCLMemory);
		SpecialReply[1]=TMCLMemory.Opcode;
		SpecialReply[2]=TMCLMemory.Type;
		SpecialReply[3]=TMCLMemory.Motor;
		SpecialReply[4]=TMCLMemory.Value.Byte[3];
		SpecialReply[5]=TMCLMemory.Value.Byte[2];
		SpecialReply[6]=TMCLMemory.Value.Byte[1];
		SpecialReply[7]=TMCLMemory.Value.Byte[0];
		SpecialReply[8]=0;
		for(i=1; i<8; i++)
			SpecialReply[8]+=SpecialReply[i];
	}
	else
	{
		for(i=1; i<8; i++) SpecialReply[i]=0;
		SpecialReply[8]=SpecialReply[0];
	}
}

/***********************************
   Funktion: ReturnFromSubroutine()

   Zweck: TMCL-Befehl RSUB
 ***********************************/
void ReturnFromSubroutine()
{
	if(TMCLStackPointer>0)
		TMCLProgramCounter = TMCLStack[--TMCLStackPointer];
}

/*********************************
   Funktion: SecurityCode()

   Zweck: TMCL-Befehl 140
          (set security code)
 *********************************/
void SecurityCode()
{
	u32 Code;

	switch(ActualCommand.Type)
	{
    	case 0:
    		eeprom_readConfigBlock(TMCM_ADDR_SECURITY_CODE, (u8 *)&Code, 4);
    		if((u32) ActualCommand.Value.Int32 == Code)
    			ActualReply.Value.Int32 = 1;
    		else
    			ActualReply.Value.Int32 = 0;
    		break;
    	case 1:
    		if(ActualCommand.Motor == 42)
    			eeprom_writeConfigBlock(TMCM_ADDR_SECURITY_CODE, (u8 *)&ActualCommand.Value.Int32, 4);
    		else
    			ActualReply.Status = REPLY_WRONG_TYPE;
    		break;
    	default:
    		ActualReply.Status = REPLY_WRONG_TYPE;
    		break;
	}
}

/*********************************
   Funktion: Stop()

   Zweck: TMCL-Befehl STOP
 *********************************/
void Stop()
{
	TMCLMode = TM_IDLE;
}

/*******************************************************************
   Funktion: StoreActualCommand

   Zweck: Ablegen des aktuellen Befehls (der in ActualCommand steht)
          im Programmspeicher (Cache und EEPROM).
********************************************************************/
void StoreActualCommand()
{
	//Antworttelegramm vorbelegen
	ActualReply.Opcode = ActualCommand.Opcode;
	ActualReply.Value.Int32 = ActualCommand.Value.Int32;

	if(TMCLMemoryPointer <= TMCL_MEM_SIZE)
	{
		eeprom_writeTMCLCommand(TMCLMemoryPointer, &ActualCommand);
		if(eeprom_verifyTMCLCommand(TMCLMemoryPointer, &ActualCommand))
		{
			TMCLMemoryPointer++;
			ActualReply.Status = REPLY_CMD_LOADED;
		}
		else ActualReply.Status = REPLY_CMD_LOAD_ERROR;
	}
	else
	{
		ActualReply.Status = REPLY_CMD_LOAD_ERROR;
		TMCLMode = TM_IDLE;
	}
}

/*********************************
   Funktion: Wait()

   Zweck: TMCL-Befehl WAIT
 *********************************/
void Wait()
{
	if(!TMCLWaitFlag)  //Initialisierung der Wartefunktion
	{
		if(ActualCommand.Value.Int32==-1)
			TMCLWaitDuration=TMCLAccu*10;
		else
			TMCLWaitDuration=ActualCommand.Value.Int32*10;
		TMCLWaitStartTime = systick_getTimer();
		TMCLWaitFlag=true;
	}

	switch(ActualCommand.Type)
	{
    	case WAIT_TICKS:
    		if(abs(systick_getTimer()-TMCLWaitStartTime)>TMCLWaitDuration)  //Zeit abgelaufen?
    			TMCLWaitFlag=false;
    		else
    			TMCLProgramCounter--;  //weiter warten (=> diesen Befehl nochmals ausf�hren)
    		break;
    	case WAIT_POS:
    		if(StepDir_getStatus(0) & STATUS_TARGET_REACHED)  //Position erreicht?
    		{
    			TMCLWaitFlag=false;
    			TMCLCPUFlags&= ~FLAG_ERROR_TIMEOUT;
    		}
    		else if(TMCLWaitDuration>0 && abs(systick_getTimer()-TMCLWaitStartTime)>TMCLWaitDuration)  //Timeout?
    		{
    			TMCLWaitFlag=false;
    			TMCLCPUFlags|= FLAG_ERROR_TIMEOUT;
    		}
    		else TMCLProgramCounter--;  //weiter warten (=> diesen Befehl nochmals ausf�hren)
    		break;
	}
}

/*********************************
   Funktion: SetCoordinate()

   Zweck: TMCL-Befehl SCO
 *********************************/
void SetCoordinate()
{
	if(ActualCommand.Motor == 0)
	{
		if(ActualCommand.Type < TMCL_COORDINATES)
		{
			TMCLCoordinate[ActualCommand.Type] = ActualCommand.Value.Int32;
		}
		else ActualReply.Status = REPLY_WRONG_TYPE;
	}
	else ActualReply.Status = REPLY_INVALID_VALUE;
}

/*********************************
   Funktion: GetCoordinate()

   Zweck: TMCL-Befehl GCO
 *********************************/
void GetCoordinate()
{
	if(ActualCommand.Motor == 0)
	{
		if(ActualCommand.Type < TMCL_COORDINATES)
		{
			ActualReply.Value.Int32 = TMCLCoordinate[ActualCommand.Type];
		}
		else ActualReply.Status = REPLY_WRONG_TYPE;
	}
	else ActualReply.Status = REPLY_INVALID_VALUE;

	if(TMCLModifyAccu)
	{
		TMCLAccu=ActualReply.Value.Int32;

		if(TMCLAccu == 0)
			TMCLCPUFlags |= FLAG_ZERO;
		else
			TMCLCPUFlags &= ~FLAG_ZERO;
	}
}

/*********************************
   Funktion: SetCoordinate()

   Zweck: TMCL-Befehl CCO
 *********************************/
void CaptureCoordinate()
{
	if(ActualCommand.Motor == 0)
	{
		if(ActualCommand.Type<TMCL_COORDINATES)
		{
			TMCLCoordinate[ActualCommand.Type] = StepDir_getActualPosition(ActualCommand.Motor);
		}
		else ActualReply.Status = REPLY_WRONG_TYPE;
	}
	else ActualReply.Status = REPLY_INVALID_VALUE;
}
